@@ -21,7 +21,7 @@ public enum DeserializationFeature implements ConfigFeature
 {
     /*
     /******************************************************
-    /* Type conversion features
+    /* Value (mostly scalar) conversion features
     /******************************************************
      */
 
@@ -33,7 +33,13 @@ public enum DeserializationFeature implements ConfigFeature
      * or {@link java.util.Collection} context) is available.
      * If enabled such values will be deserialized as {@link java.math.BigDecimal}s;
      * if disabled, will be deserialized as {@link Double}s.
-     * <p>
+     *<p>
+     * NOTE: one aspect of {@link java.math.BigDecimal} handling that may need
+     * configuring is whether trailing zeroes are trimmed:
+     * {@link com.fasterxml.jackson.databind.node.JsonNodeFactory} has
+     * {@link com.fasterxml.jackson.databind.node.JsonNodeFactory#withExactBigDecimals} for
+     * changing default behavior (default is for trailing zeroes to be trimmed).
+     *<p>
      * Feature is disabled by default, meaning that "untyped" floating
      * point numbers will by default be deserialized as {@link Double}s
      * (choice is for performance reason -- BigDecimals are slower than
@@ -53,40 +59,49 @@ public enum DeserializationFeature implements ConfigFeature
      * which is either {@link Integer}, {@link Long} or
      * {@link java.math.BigInteger}, depending on number of digits.
      * <p>
-     * Feature is disabled by default, meaning that "untyped" floating
-     * point numbers will by default be deserialized using whatever
+     * Feature is disabled by default, meaning that "untyped" integral
+     * numbers will by default be deserialized using whatever
      * is the most compact integral type, to optimize efficiency.
      */
     USE_BIG_INTEGER_FOR_INTS(false),
 
-    // [JACKSON-652]
+    /**
+     * Feature that determines how "small" JSON integral (non-floating-point)
+     * numbers -- ones that fit in 32-bit signed integer (`int`) -- are bound
+     * when target type is loosely typed as {@link Object} or {@link Number}
+     * (or within untyped {@link java.util.Map} or {@link java.util.Collection} context).
+     * If enabled, such values will be deserialized as {@link java.lang.Long};
+     * if disabled, they will be deserialized as "smallest" available type,
+     * {@link Integer}.
+     * In addition, if enabled, trying to bind values that do not fit in {@link java.lang.Long}
+     * will throw a {@link com.fasterxml.jackson.core.JsonProcessingException}.
+     *<p>
+     * Note: if {@link #USE_BIG_INTEGER_FOR_INTS} is enabled, it has precedence
+     * over this setting, forcing use of {@link java.math.BigInteger} for all
+     * integral values.
+     *<p>
+     * Feature is disabled by default, meaning that "untyped" integral
+     * numbers will by default be deserialized using {@link java.lang.Integer}
+     * if value fits.
+     * 
+     * @since 2.6
+     */
+    USE_LONG_FOR_INTS(false),
+    
     /**
      * Feature that determines whether JSON Array is mapped to
-     * <code>Object[]</code> or <code>List&lt;Object></code> when binding
+     * <code>Object[]</code> or <code>List&lt;Object&gt;</code> when binding
      * "untyped" objects (ones with nominal type of <code>java.lang.Object</code>).
-     * If true, binds as <code>Object[]</code>; if false, as <code>List&lt;Object></code>.
+     * If true, binds as <code>Object[]</code>; if false, as <code>List&lt;Object&gt;</code>.
      *<p>
      * Feature is disabled by default, meaning that JSON arrays are bound as
      * {@link java.util.List}s.
      */
     USE_JAVA_ARRAY_FOR_JSON_ARRAY(false),
-    
-    /**
-     * Feature that determines standard deserialization mechanism used for
-     * Enum values: if enabled, Enums are assumed to have been serialized  using
-     * return value of <code>Enum.toString()</code>;
-     * if disabled, return value of <code>Enum.name()</code> is assumed to have been used.
-     *<p>
-     * Note: this feature should usually have same value
-     * as {@link SerializationFeature#WRITE_ENUMS_USING_TO_STRING}.
-     *<p>
-     * Feature is disabled by default.
-     */
-    READ_ENUMS_USING_TO_STRING(false),
-    
+
     /*
     /******************************************************
-     *  Error handling features
+    /* Error handling features
     /******************************************************
      */
 
@@ -134,7 +149,7 @@ public enum DeserializationFeature implements ConfigFeature
     /**
      * Feature that determines what happens when type of a polymorphic
      * value (indicated for example by {@link com.fasterxml.jackson.annotation.JsonTypeInfo})
-     * can not be found (missing) or resolved (invalid class name, unmappable id);
+     * cannot be found (missing) or resolved (invalid class name, unmappable id);
      * if enabled, an exception ir thrown; if false, null value is used instead.
      *<p>
      * Feature is enabled by default so that exception is thrown for missing or invalid
@@ -172,6 +187,90 @@ public enum DeserializationFeature implements ConfigFeature
      */
     FAIL_ON_IGNORED_PROPERTIES(false),
 
+    /**
+     * Feature that determines what happens if an Object Id reference is encountered
+     * that does not refer to an actual Object with that id ("unresolved Object Id"):
+     * either an exception is thrown (<code>true</code>), or a null object is used
+     * instead (<code>false</code>).
+     * Note that if this is set to <code>false</code>, no further processing is done;
+     * specifically, if reference is defined via setter method, that method will NOT
+     * be called.
+     *<p>
+     * Feature is enabled by default, so that unknown Object Ids will result in an
+     * exception being thrown, at the end of deserialization.
+     * 
+     * @since 2.5
+     */
+    FAIL_ON_UNRESOLVED_OBJECT_IDS(true),
+
+    /**
+     * Feature that determines what happens if one or more Creator properties (properties
+     * bound to parameters of Creator method (constructor or static factory method))
+     * are missing value to bind to from content.
+     * If enabled, such missing values result in a {@link JsonMappingException} being
+     * thrown with information on the first one (by index) of missing properties.
+     * If disabled, and if property is NOT marked as required,
+     * missing Creator properties are filled
+     * with <code>null values</code> provided by deserializer for the type of parameter
+     * (usually null for Object types, and default value for primitives; but redefinable
+     * via custom deserializers).
+     *<p>
+     * Note that having an injectable value counts as "not missing".
+     *<p>
+     * Feature is disabled by default, so that no exception is thrown for missing creator
+     * property values, unless they are explicitly marked as `required`.
+     * 
+     * @since 2.6
+     */
+    FAIL_ON_MISSING_CREATOR_PROPERTIES(false),
+
+    /**
+      * Feature that determines what happens if one or more Creator properties (properties
+      * bound to parameters of Creator method (constructor or static factory method))
+      * are bound to null values - either from the JSON or as a default value. This
+      * is useful if you want to avoid nulls in your codebase, and particularly useful
+      * if you are using Java or Scala optionals for non-mandatory fields.
+      * Feature is disabled by default, so that no exception is thrown for missing creator
+      * property values, unless they are explicitly marked as `required`.
+      *
+      * @since 2.8
+      */
+    FAIL_ON_NULL_CREATOR_PROPERTIES(false),
+
+    /**
+     * Feature that determines what happens when a property annotated with
+     * {@link com.fasterxml.jackson.annotation.JsonTypeInfo.As#EXTERNAL_PROPERTY} is missing,
+     * but associated type id is available. If enabled, {@link JsonMappingException} is always
+     * thrown when property value is missing (if type id does exist);
+     * if disabled, exception is only thrown if property is marked as `required`.
+     *<p>
+     * Feature is enabled by default, so that exception is thrown when a subtype property is
+     * missing.
+     *
+     * @since 2.9
+     */
+    FAIL_ON_MISSING_EXTERNAL_TYPE_ID_PROPERTY(true),
+
+    /**
+     * Feature that determines behaviour for data-binding after binding the root value.
+     * If feature is enabled, one more call to
+     * {@link com.fasterxml.jackson.core.JsonParser#nextToken} is made to ensure that
+     * no more tokens are found (and if any is found,
+     * {@link com.fasterxml.jackson.databind.exc.MismatchedInputException} is thrown); if
+     * disabled, no further checks are made.
+     *<p>
+     * Feature could alternatively be called <code>READ_FULL_STREAM</code>, since it
+     * effectively verifies that input stream contains only as much data as is needed
+     * for binding the full value, and nothing more (except for possible ignorable
+     * white space or comments, if supported by data format).
+     *<p>
+     * Feature is disabled by default (so that no check is made for possible trailing
+     * token(s)) for backwards compatibility reasons.
+     *
+     * @since 2.9
+     */
+    FAIL_ON_TRAILING_TOKENS(false),
+    
     /**
      * Feature that determines whether Jackson code should catch
      * and wrap {@link Exception}s (but never {@link Error}s!)
@@ -188,7 +287,7 @@ public enum DeserializationFeature implements ConfigFeature
      * Feature is enabled by default.
      */
     WRAP_EXCEPTIONS(true),
-    
+
     /*
     /******************************************************
     /* Structural conversion features
@@ -239,17 +338,65 @@ public enum DeserializationFeature implements ConfigFeature
     
     /**
      * Feature that can be enabled to allow JSON empty String
-     * value ("") to be bound to POJOs as null.
-     * If disabled, standard POJOs can only be bound from JSON null or
+     * value ("") to be bound as `null` for POJOs and other structured
+     * values ({@link java.util.Map}s, {@link java.util.Collection}s).
+     * If disabled, standard POJOs can only be bound from JSON `null` or
      * JSON Object (standard meaning that no custom deserializers or
      * constructors are defined; both of which can add support for other
-     * kinds of JSON values); if enable, empty JSON String can be taken
+     * kinds of JSON values); if enabled, empty JSON String can be taken
      * to be equivalent of JSON null.
      *<p>
-     * Feature is enabled by default.
+     * NOTE: this does NOT apply to scalar values such as booleans and numbers;
+     * whether they can be coerced depends on
+     * {@link MapperFeature#ALLOW_COERCION_OF_SCALARS}.
+     *<p>
+     * Feature is disabled by default.
      */
     ACCEPT_EMPTY_STRING_AS_NULL_OBJECT(false),
-    
+
+    /**
+     * Feature that can be enabled to allow empty JSON Array
+     * value (that is, <code>[ ]</code>) to be bound to POJOs (and
+     * with 2.9, other values too) as `null`.
+     * If disabled, standard POJOs can only be bound from JSON `null` or
+     * JSON Object (standard meaning that no custom deserializers or
+     * constructors are defined; both of which can add support for other
+     * kinds of JSON values); if enabled, empty JSON Array will be taken
+     * to be equivalent of JSON null.
+     *<p>
+     * Feature is disabled by default.
+     * 
+     * @since 2.5
+     */
+    ACCEPT_EMPTY_ARRAY_AS_NULL_OBJECT(false),
+
+    /**
+     * Feature that determines whether coercion from JSON floating point
+     * number (anything with command (`.`) or exponent portion (`e` / `E'))
+     * to an expected integral number (`int`, `long`, `java.lang.Integer`, `java.lang.Long`,
+     * `java.math.BigDecimal`) is allowed or not.
+     * If enabled, coercion truncates value; if disabled, a {@link JsonMappingException}
+     * will be thrown.
+     *<p>
+     * Feature is enabled by default.
+     * 
+     * @since 2.6
+     */
+    ACCEPT_FLOAT_AS_INT(true),
+
+    /**
+     * Feature that determines standard deserialization mechanism used for
+     * Enum values: if enabled, Enums are assumed to have been serialized  using
+     * return value of <code>Enum.toString()</code>;
+     * if disabled, return value of <code>Enum.name()</code> is assumed to have been used.
+     *<p>
+     * Note: this feature should usually have same value
+     * as {@link SerializationFeature#WRITE_ENUMS_USING_TO_STRING}.
+     *<p>
+     * Feature is disabled by default.
+     */
+    READ_ENUMS_USING_TO_STRING(false),
+
     /**
      * Feature that allows unknown Enum values to be parsed as null values. 
      * If disabled, unknown Enum values will throw exceptions.
@@ -265,6 +412,18 @@ public enum DeserializationFeature implements ConfigFeature
      */
     READ_UNKNOWN_ENUM_VALUES_AS_NULL(false),
 
+    /**
+     * Feature that allows unknown Enum values to be ignored and a predefined value specified through
+     * {@link com.fasterxml.jackson.annotation.JsonEnumDefaultValue @JsonEnumDefaultValue} annotation.
+     * If disabled, unknown Enum values will throw exceptions.
+     * If enabled, but no predefined default Enum value is specified, an exception will be thrown as well.
+     *<p>
+     * Feature is disabled by default.
+     *
+     * @since 2.8
+     */
+    READ_UNKNOWN_ENUM_VALUES_USING_DEFAULT_VALUE(false),
+
     /**
      * Feature that controls whether numeric timestamp values are expected
      * to be written using nanosecond timestamps (enabled) or not (disabled),
@@ -289,6 +448,18 @@ public enum DeserializationFeature implements ConfigFeature
      * If enabled, contextual <code>TimeZone</code> will essentially override any other
      * TimeZone information; if disabled, it will only be used if value itself does not
      * contain any TimeZone information.
+     *<p>
+     * Note that exact behavior depends on date/time types in question; and specifically
+     * JDK type of {@link java.util.Date} does NOT have in-built timezone information
+     * so this setting has no effect.
+     * Further, while {@link java.util.Calendar} does have this information basic
+     * JDK {@link java.text.SimpleDateFormat} is unable to retain parsed zone information,
+     * and as a result, {@link java.util.Calendar} will always get context timezone
+     * adjustment regardless of this setting.
+     *<p>
+     *<p>
+     * Taking above into account, this feature is supported only by extension modules for
+     * Joda and Java 8 date/tyime datatypes.
      * 
      * @since 2.2
      */
@@ -319,14 +490,19 @@ public enum DeserializationFeature implements ConfigFeature
     ;
 
     private final boolean _defaultState;
+    private final int _mask;
     
     private DeserializationFeature(boolean defaultState) {
         _defaultState = defaultState;
+        _mask = (1 << ordinal());
     }
 
     @Override
     public boolean enabledByDefault() { return _defaultState; }
 
     @Override
-    public int getMask() { return (1 << ordinal()); }
-}
\ No newline at end of file
+    public int getMask() { return _mask; }
+
+    @Override
+    public boolean enabledIn(int flags) { return (flags & _mask) != 0; }
+}
