@@ -1,15 +1,18 @@
 package com.fasterxml.jackson.databind;
 
 import java.lang.reflect.Modifier;
+import java.util.List;
 
 import com.fasterxml.jackson.core.type.ResolvedType;
+import com.fasterxml.jackson.databind.type.TypeBindings;
+import com.fasterxml.jackson.databind.type.TypeFactory;
 
 /**
  * Base class for type token classes used both to contain information
  * and as keys for deserializers.
  *<p>
  * Instances can (only) be constructed by
- * <code>com.fasterxml.jackson.databind.TypeFactory</code>.
+ * <code>com.fasterxml.jackson.databind.type.TypeFactory</code>.
  *<p>
  * Since 2.2 this implements {@link java.lang.reflect.Type} to allow
  * it to be pushed through interfaces that only expose that type.
@@ -79,7 +82,21 @@ public abstract class JavaType
         _typeHandler = typeHandler;
         _asStatic = asStatic;
     }
-    
+
+    /**
+     * Copy-constructor used when refining/upgrading type instances.
+     *
+     * @since 2.7
+     */
+    protected JavaType(JavaType base) 
+    {
+        _class = base._class;
+        _hash = base._hash;
+        _valueHandler = base._valueHandler;
+        _typeHandler = base._typeHandler;
+        _asStatic = base._asStatic;
+    }
+
     /**
      * "Copy method" that will construct a new instance that is identical to
      * this instance, except that it will have specified type handler assigned.
@@ -89,16 +106,16 @@ public abstract class JavaType
     public abstract JavaType withTypeHandler(Object h);
 
     /**
-     * "Copy method" that will construct a new instance that is identical to
-     * this instance, except that its content type will have specified
-     * type handler assigned.
+     * Mutant factory method that will construct a new instance that is identical to
+     * this instance, except that it will have specified content type (element type
+     * for arrays, value type for Maps and so forth) handler assigned.
      * 
-     * @return Newly created type instance
+     * @return Newly created type instance, with given 
      */
     public abstract JavaType withContentTypeHandler(Object h);
 
     /**
-     * "Copy method" that will construct a new instance that is identical to
+     * Mutant factory method that will construct a new instance that is identical to
      * this instance, except that it will have specified value handler assigned.
      * 
      * @return Newly created type instance
@@ -106,13 +123,50 @@ public abstract class JavaType
     public abstract JavaType withValueHandler(Object h);
 
     /**
-     * "Copy method" that will construct a new instance that is identical to
+     * Mutant factory method that will construct a new instance that is identical to
      * this instance, except that it will have specified content value handler assigned.
      * 
      * @return Newly created type instance
      */
     public abstract JavaType withContentValueHandler(Object h);
 
+    /**
+     * Mutant factory method that will try to copy handlers that the specified
+     * source type instance had, if any; this must be done recursively where
+     * necessary (as content types may be structured).
+     *
+     * @since 2.8.4
+     */
+    public JavaType withHandlersFrom(JavaType src) {
+        JavaType type = this;
+        Object h = src.getTypeHandler();
+        if (h != _typeHandler) {
+            type = type.withTypeHandler(h);
+        }
+        h = src.getValueHandler();
+        if (h != _valueHandler) {
+            type = type.withValueHandler(h);
+        }
+        return type;
+    }
+
+    /**
+     * Mutant factory method that may be called on structured types
+     * that have a so-called content type (element of arrays, value type
+     * of Maps, referenced type of referential types),
+     * and will construct a new instance that is identical to
+     * this instance, except that it has specified content type, instead of current
+     * one. If content type is already set to given type, <code>this</code> is returned.
+     * If type does not have a content type (which is the case with
+     * <code>SimpleType</code>), {@link IllegalArgumentException}
+     * will be thrown.
+     * 
+     * @return Newly created type instance
+     *
+     * @since 2.7
+     */
+    public abstract JavaType withContentType(JavaType contentType);
+
     /**
      * Method that can be called to get a type instance that indicates
      * that values of the type should be handled using "static typing" for purposes
@@ -131,84 +185,38 @@ public abstract class JavaType
     /* Type coercion fluent factory methods
     /**********************************************************
      */
+
+    /**
+     * Mutant factory method that will try to create and return a sub-type instance
+     * for known parameterized types; for other types will return `null` to indicate
+     * that no just refinement makes necessary sense, without trying to detect
+     * special status through implemented interfaces.
+     *
+     * @since 2.7
+     */
+    public abstract JavaType refine(Class<?> rawType, TypeBindings bindings,
+            JavaType superClass, JavaType[] superInterfaces);
     
     /**
-     * Method that can be called to do a "narrowing" conversions; that is,
-     * to return a type with a raw class that is assignable to the raw
-     * class of this type. If this is not possible, an
-     * {@link IllegalArgumentException} is thrown.
-     * If class is same as the current raw class, instance itself is
-     * returned.
-     */
-    public JavaType narrowBy(Class<?> subclass)
-    {
-        // First: if same raw class, just return this instance
-        if (subclass == _class) { return this; }
-        // Otherwise, ensure compatibility
-        _assertSubclass(subclass, _class);
-        JavaType result = _narrow(subclass);
-
-        // TODO: these checks should NOT actually be needed; above should suffice:
-        if (_valueHandler != result.<Object>getValueHandler()) {
-            result = result.withValueHandler(_valueHandler);
-        }
-        if (_typeHandler != result.<Object>getTypeHandler()) {
-            result = result.withTypeHandler(_typeHandler);
-        }
-        return result;
-    }
-
-    /**
-     * More efficient version of {@link #narrowBy}, called by
-     * internal framework in cases where compatibility checks
-     * are to be skipped.
+     * Legacy method used for forcing sub-typing of this type into
+     * type specified by specific type erasure.
+     * Deprecated as of 2.7 as such specializations really ought to
+     * go through {@link TypeFactory}, not directly via {@link JavaType}.
+     *
+     * @since 2.7
      */
+    @Deprecated
     public JavaType forcedNarrowBy(Class<?> subclass)
     {
         if (subclass == _class) { // can still optimize for simple case
             return this;
         }
-        JavaType result = _narrow(subclass);
-        // TODO: these checks should NOT actually be needed; above should suffice:
-        if (_valueHandler != result.<Object>getValueHandler()) {
-            result = result.withValueHandler(_valueHandler);
-        }
-        if (_typeHandler != result.<Object>getTypeHandler()) {
-            result = result.withTypeHandler(_typeHandler);
-        }
-        return result;
-    }
-
-    /**
-     * Method that can be called to do a "widening" conversions; that is,
-     * to return a type with a raw class that could be assigned from this
-     * type.
-     * If such conversion is not possible, an
-     * {@link IllegalArgumentException} is thrown.
-     * If class is same as the current raw class, instance itself is
-     * returned.
-     */
-    public JavaType widenBy(Class<?> superclass) {
-        // First: if same raw class, just return this instance
-        if (superclass == _class) { return this; }
-        // Otherwise, ensure compatibility
-        _assertSubclass(_class, superclass);
-        return _widen(superclass);
+        return  _narrow(subclass);
     }
 
+    @Deprecated // since 2.7
     protected abstract JavaType _narrow(Class<?> subclass);
 
-    /**
-     *<p>
-     * Default implementation is just to call {@link #_narrow}, since
-     * underlying type construction is usually identical
-     */
-    protected JavaType _widen(Class<?> superclass) { return _narrow(superclass); }
-
-    public abstract JavaType narrowContentsBy(Class<?> contentClass);
-
-    public abstract JavaType widenContentsBy(Class<?> contentClass);
-    
     /*
     /**********************************************************
     /* Implementation of ResolvedType API
@@ -226,6 +234,31 @@ public abstract class JavaType
     @Override
     public final boolean hasRawClass(Class<?> clz) { return _class == clz; }
 
+    /**
+     * Accessor that allows determining whether {@link #getContentType()} should
+     * return a non-null value (that is, there is a "content type") or not.
+     * True if {@link #isContainerType()} or {@link #isReferenceType()} return true.
+     *
+     * @since 2.8
+     */
+    public boolean hasContentType() {
+        return true;
+    }
+
+    /**
+     * @since 2.6
+     */
+    public final boolean isTypeOrSubTypeOf(Class<?> clz) {
+        return (_class == clz) || clz.isAssignableFrom(_class);
+    }
+
+    /**
+     * @since 2.9
+     */
+    public final boolean isTypeOrSuperTypeOf(Class<?> clz) {
+        return (_class == clz) || _class.isAssignableFrom(clz);
+    }
+
     @Override
     public boolean isAbstract() {
         return Modifier.isAbstract(_class.getModifiers());
@@ -255,7 +288,13 @@ public abstract class JavaType
     public boolean isArrayType() { return false; }
 
     @Override
-    public final boolean isEnumType() { return _class.isEnum(); }
+    public final boolean isEnumType() {
+        // 29-Sep-2019, tatu: `Class.isEnum()` not enough to detect custom subtypes,
+        //   but for some reason this fix will break couple of unit tests:
+        // See [databind#2480]:
+//        return ClassUtil.isEnumType(_class);
+        return _class.isEnum();
+    }
 
     @Override
     public final boolean isInterface() { return _class.isInterface(); }
@@ -289,6 +328,17 @@ public abstract class JavaType
     @Override
     public boolean isMapLikeType() { return false; }
 
+    /**
+     * Convenience method, short-hand for
+     *<code>
+     *   getRawClass() == Object.class
+     *</code>
+     * and used to figure if we basically have "untyped" type object.
+     *
+     * @since 2.5
+     */
+    public final boolean isJavaLangObject() { return _class == Object.class; }
+
     /**
      * Accessor for checking whether handlers for dealing with values of
      * this type should use static typing (as opposed to dynamic typing).
@@ -298,7 +348,7 @@ public abstract class JavaType
      * @since 2.2
      */
     public final boolean useStaticType() { return _asStatic; }
-    
+
     /*
     /**********************************************************
     /* Public API, type parameter access; pass-through
@@ -314,14 +364,91 @@ public abstract class JavaType
     @Override
     public JavaType getContentType() { return null; }
 
-    @Override
-    public int containedTypeCount() { return 0; }
+    @Override // since 2.6
+    public JavaType getReferencedType() { return null; }
 
     @Override
-    public JavaType containedType(int index) { return null; }
+    public abstract int containedTypeCount();
 
     @Override
-    public String containedTypeName(int index) { return null; }
+    public abstract JavaType containedType(int index);
+       
+    @Deprecated // since 2.7
+    @Override
+    public abstract String containedTypeName(int index);
+
+    @Deprecated // since 2.7
+    @Override
+    public Class<?> getParameterSource() {
+        return null;
+    }
+
+    /*
+    /**********************************************************
+    /* Extended API beyond ResolvedType
+    /**********************************************************
+     */
+    
+    // NOTE: not defined in Resolved type
+    /**
+     * Convenience method that is functionally same as:
+     *<code>
+     * JavaType t = containedType(index);
+     * if (t == null) {
+     *    t = TypeFactory.unknownType();
+     * }
+     *</code>
+     * and typically used to eliminate need for null checks for common case
+     * where we just want to check if containedType is available first; and
+     * if not, use "unknown type" (which translates to <code>java.lang.Object</code>
+     * basically).
+     *
+     * @since 2.5
+     */
+    public JavaType containedTypeOrUnknown(int index) {
+        JavaType t = containedType(index);
+        return (t == null)  ? TypeFactory.unknownType() : t;
+    }
+
+    /**
+     * @since 2.7
+     */
+    public abstract TypeBindings getBindings();
+
+    /**
+     * Method that may be called to find representation of given type
+     * within type hierarchy of this type: either this type (if this
+     * type has given erased type), one of its supertypes that has the
+     * erased types, or null if target is neither this type or any of its
+     * supertypes.
+     *
+     * @since 2.7
+     */
+    public abstract JavaType findSuperType(Class<?> erasedTarget);
+
+    /**
+     * Accessor for finding fully resolved parent class of this type,
+     * if it has one; null if not.
+     *
+     * @since 2.7
+     */
+    public abstract JavaType getSuperClass();
+
+    /**
+     * Accessor for finding fully resolved interfaces this type implements,
+     * if any; empty array if none.
+     *
+     * @since 2.7
+     */
+    public abstract List<JavaType> getInterfaces();
+
+    /**
+     * Method that may be used to find paramaterization this type has for
+     * given type-erased generic target type.
+     *
+     * @since 2.7
+     */
+    public abstract JavaType[] findTypeParameters(Class<?> expType);
 
     /*
     /**********************************************************
@@ -341,6 +468,33 @@ public abstract class JavaType
     @SuppressWarnings("unchecked")
     public <T> T getTypeHandler() { return (T) _typeHandler; }
 
+    /**
+     * @since 2.7
+     */
+    public Object getContentValueHandler() { return null; }
+
+    /**
+     * @since 2.7
+     */
+    public Object getContentTypeHandler() { return null; }    
+
+    /**
+     * @since 2.6
+     */
+    public boolean hasValueHandler() { return _valueHandler != null; }
+
+    /**
+     * Helper method that checks whether this type, or its (optional) key
+     * or content type has {@link #getValueHandler} or {@link #getTypeHandler()};
+     * that is, are there any non-standard handlers associated with this
+     * type object.
+     *
+     * @since 2.8
+     */
+    public boolean hasHandlers() {
+        return (_typeHandler != null) || (_valueHandler != null);
+    }
+    
     /*
     /**********************************************************
     /* Support for producing signatures
@@ -396,18 +550,6 @@ public abstract class JavaType
      * call chaining
      */
     public abstract StringBuilder getErasedSignature(StringBuilder sb);
-    
-    /*
-    /**********************************************************
-    /* Helper methods
-    /**********************************************************
-     */
-
-    protected void _assertSubclass(Class<?> subclass, Class<?> superClass) {
-        if (!_class.isAssignableFrom(subclass)) {
-            throw new IllegalArgumentException("Class "+subclass.getName()+" is not assignable to "+_class.getName());
-        }
-    }
 
     /*
     /**********************************************************
