@@ -195,15 +195,15 @@ public enum MapperFeature implements ConfigFeature
     INFER_PROPERTY_MUTATORS(true),
 
     /**
-     * Feature that determines handling of <code>java.beans.ConstructorProperties<code>
+     * Feature that determines handling of {@code java.beans.ConstructorProperties}
      * annotation: when enabled, it is considered as alias of
      * {@link com.fasterxml.jackson.annotation.JsonCreator}, to mean that constructor
      * should be considered a property-based Creator; when disabled, only constructor
      * parameter name information is used, but constructor is NOT considered an explicit
      * Creator (although may be discovered as one using other annotations or heuristics).
      *<p>
-     * Feature is mostly used to help interoperability with frameworks like Lombok
-     * that may automatically generate <code>ConstructorProperties</code> annotation
+     * Feature is mostly used to help inter-operability with frameworks like Lombok
+     * that may automatically generate {@code ConstructorProperties} annotation
      * but without necessarily meaning that constructor should be used as Creator
      * for deserialization.
      *<p>
@@ -286,6 +286,20 @@ public enum MapperFeature implements ConfigFeature
      */
     USE_STATIC_TYPING(false),
 
+    /**
+     * Feature that specifies whether the declared base type of a polymorphic value
+     * is to be used as the "default" implementation, if no explicit default class
+     * is specified via {@code @JsonTypeInfo.defaultImpl} annotation.
+     *<p>
+     * Note that feature only has effect on deserialization of regular polymorphic properties:
+     * it does NOT affect non-polymorphic cases, and is unlikely to work with Default Typing.
+     *<p>
+     * Feature is disabled by default for backwards compatibility.
+     *
+     * @since 2.10
+     */
+    USE_BASE_TYPE_AS_DEFAULT_IMPL(false),
+
     /*
     /******************************************************
     /* View-related features
@@ -347,7 +361,7 @@ public enum MapperFeature implements ConfigFeature
      *<p>
      * Note that there is additional performance overhead since incoming property
      * names need to be lower-cased before comparison, for cases where there are upper-case
-     * letters. Overhead for names that are already lower-case should be negligible however.
+     * letters. Overhead for names that are already lower-case should be negligible.
      *<p>
      * Feature is disabled by default.
      * 
@@ -359,7 +373,7 @@ public enum MapperFeature implements ConfigFeature
     /**
      * Feature that determines if Enum deserialization should be case sensitive or not.
      * If enabled, Enum deserialization will ignore case, that is, case of incoming String
-     * value and enum id (dependant on other settings, either `name()`, `toString()`, or
+     * value and enum id (depending on other settings, either `name()`, `toString()`, or
      * explicit override) do not need to match.
      * <p>
      * Feature is disabled by default.
@@ -368,6 +382,20 @@ public enum MapperFeature implements ConfigFeature
      */
     ACCEPT_CASE_INSENSITIVE_ENUMS(false),
 
+    /**
+     * Feature that permits parsing some enumerated text-based value types but ignoring the case
+     * of the values on deserialization: for example, date/time type deserializers.
+     * Support for this feature depends on deserializer implementations using it.
+     *<p>
+     * Note, however, that regular {@code Enum} types follow {@link #ACCEPT_CASE_INSENSITIVE_ENUMS}
+     * setting instead.
+     *<p>
+     * Feature is disabled by default.
+     * 
+     * @since 2.10
+     */
+    ACCEPT_CASE_INSENSITIVE_VALUES(false),
+
     /**
      * Feature that can be enabled to make property names be
      * overridden by wrapper name (usually detected with annotations
@@ -468,9 +496,8 @@ public enum MapperFeature implements ConfigFeature
      * merging is skipped and new value is created (<code>true</code>) or
      * an exception is thrown (false).
      *<p>
-     * Feature is disabled by default since non-mergeable property types are ignored
-     * even if defaults call for merging, and usually explicit per-type or per-property
-     * settings for such types should result in an exception.
+     * Feature is enabled by default, to allow use of merge defaults even in presence
+     * of some unmergeable properties.
      *
      * @since 2.9
      */
