@@ -526,7 +526,12 @@ public abstract class SettableBeanProperty
         if (_valueTypeDeserializer != null) {
             return _valueDeserializer.deserializeWithType(p, ctxt, _valueTypeDeserializer);
         }
-        return _valueDeserializer.deserialize(p, ctxt);
+        // 04-May-2018, tatu: [databind#2023] Coercion from String (mostly) can give null
+        Object value =  _valueDeserializer.deserialize(p, ctxt);
+        if (value == null) {
+            value = _nullProvider.getNullValue(ctxt);
+        }
+        return value;
     }
 
     /**
@@ -551,7 +556,15 @@ public abstract class SettableBeanProperty
                             getName()));
 //            return _valueDeserializer.deserializeWithType(p, ctxt, _valueTypeDeserializer);
         }
-        return _valueDeserializer.deserialize(p, ctxt, toUpdate);
+        // 04-May-2018, tatu: [databind#2023] Coercion from String (mostly) can give null
+        Object value = _valueDeserializer.deserialize(p, ctxt, toUpdate);
+        if (value == null) {
+            if (NullsConstantProvider.isSkipper(_nullProvider)) {
+                return toUpdate;
+            }
+            value = _nullProvider.getNullValue(ctxt);
+        }
+        return value;
     }
 
     /*
@@ -574,7 +587,7 @@ public abstract class SettableBeanProperty
                     .append(getType())
                     .append("; actual type: ")
                     .append(actType).append(")");
-            String origMsg = e.getMessage();
+            String origMsg = ClassUtil.exceptionMessage(e);
             if (origMsg != null) {
                 msg.append(", problem: ")
                     .append(origMsg);
@@ -595,7 +608,7 @@ public abstract class SettableBeanProperty
         ClassUtil.throwIfRTE(e);
         // let's wrap the innermost problem
         Throwable th = ClassUtil.getRootCause(e);
-        throw JsonMappingException.from(p, th.getMessage(), th);
+        throw JsonMappingException.from(p, ClassUtil.exceptionMessage(th), th);
     }
 
     @Deprecated // since 2.7
