@@ -1,8 +1,6 @@
 package com.fasterxml.jackson.databind.deser.impl;
 
-import java.lang.reflect.AnnotatedElement;
 import java.lang.reflect.Member;
-import java.lang.reflect.Type;
 import java.util.*;
 
 import com.fasterxml.jackson.databind.*;
@@ -68,8 +66,6 @@ public class CreatorCollector {
 
     protected SettableBeanProperty[] _propertyBasedArgs;
 
-    protected AnnotatedParameter _incompleteParameter;
-
     /*
     /**********************************************************
     /* Life-cycle
@@ -83,22 +79,18 @@ public class CreatorCollector {
                 .isEnabled(MapperFeature.OVERRIDE_PUBLIC_ACCESS_MODIFIERS);
     }
 
-    public ValueInstantiator constructValueInstantiator(
-            DeserializationConfig config) {
-        final JavaType delegateType = _computeDelegateType(
+    public ValueInstantiator constructValueInstantiator(DeserializationContext ctxt)
+        throws JsonMappingException
+    {
+        final DeserializationConfig config = ctxt.getConfig();
+        final JavaType delegateType = _computeDelegateType(ctxt,
                 _creators[C_DELEGATE], _delegateArgs);
-        final JavaType arrayDelegateType = _computeDelegateType(
+        final JavaType arrayDelegateType = _computeDelegateType(ctxt,
                 _creators[C_ARRAY_DELEGATE], _arrayDelegateArgs);
         final JavaType type = _beanDesc.getType();
 
-        // 11-Jul-2016, tatu: Earlier optimization by replacing the whole
-        // instantiator did not
-        // work well, so let's replace by lower-level check:
-        AnnotatedWithParams defaultCtor = StdTypeConstructor
-                .tryToOptimize(_creators[C_DEFAULT]);
-
         StdValueInstantiator inst = new StdValueInstantiator(config, type);
-        inst.configureFromObjectSettings(defaultCtor, _creators[C_DELEGATE],
+        inst.configureFromObjectSettings(_creators[C_DEFAULT], _creators[C_DELEGATE],
                 delegateType, _delegateArgs, _creators[C_PROPS],
                 _propertyBasedArgs);
         inst.configureFromArraySettings(_creators[C_ARRAY_DELEGATE],
@@ -108,7 +100,6 @@ public class CreatorCollector {
         inst.configureFromLongCreator(_creators[C_LONG]);
         inst.configureFromDoubleCreator(_creators[C_DOUBLE]);
         inst.configureFromBooleanCreator(_creators[C_BOOLEAN]);
-        inst.configureIncompleteParameter(_incompleteParameter);
         return inst;
     }
 
@@ -184,8 +175,8 @@ public class CreatorCollector {
                     Integer old = names.put(name, Integer.valueOf(i));
                     if (old != null) {
                         throw new IllegalArgumentException(String.format(
-                                "Duplicate creator property \"%s\" (index %s vs %d)",
-                                name, old, i));
+                                "Duplicate creator property \"%s\" (index %s vs %d) for type %s ",
+                                name, old, i, ClassUtil.nameOf(_beanDesc.getBeanClass())));
                     }
                 }
             }
@@ -193,12 +184,6 @@ public class CreatorCollector {
         }
     }
 
-    public void addIncompeteParameter(AnnotatedParameter parameter) {
-        if (_incompleteParameter == null) {
-            _incompleteParameter = parameter;
-        }
-    }
-
     /*
     /**********************************************************
     /* Accessors
@@ -232,8 +217,10 @@ public class CreatorCollector {
     /**********************************************************
      */
 
-    private JavaType _computeDelegateType(AnnotatedWithParams creator,
-            SettableBeanProperty[] delegateArgs) {
+    private JavaType _computeDelegateType(DeserializationContext ctxt,
+            AnnotatedWithParams creator, SettableBeanProperty[] delegateArgs)
+        throws JsonMappingException
+    {
         if (!_hasNonDefaultCreator || (creator == null)) {
             return null;
         }
@@ -247,7 +234,28 @@ public class CreatorCollector {
                 }
             }
         }
-        return creator.getParameterType(ix);
+        final DeserializationConfig config = ctxt.getConfig();
+
+        // 03-May-2018, tatu: need to check possible annotation-based
+        //   custom deserializer [databind#2012],
+        //   type refinement(s) [databind#2016]. 
+        JavaType baseType = creator.getParameterType(ix);
+        AnnotationIntrospector intr = config.getAnnotationIntrospector();
+        if (intr != null) {
+            AnnotatedParameter delegate = creator.getParameter(ix);
+            
+            // First: custom deserializer(s):
+            Object deserDef = intr.findDeserializer(delegate);
+            if (deserDef != null) {
+                JsonDeserializer<Object> deser = ctxt.deserializerInstance(delegate, deserDef);
+                baseType = baseType.withValueHandler(deser);
+            } else {
+                // Second: type refinement(s), if no explicit deserializer was located
+                baseType = intr.refineDeserializationType(config,
+                        delegate, baseType);
+            }
+        }
+        return baseType;
     }
 
     private <T extends AnnotatedMember> T _fixAccess(T member) {
@@ -328,173 +336,7 @@ public class CreatorCollector {
      * @since 2.8.1
      */
     protected boolean _isEnumValueOf(AnnotatedWithParams creator) {
-        return creator.getDeclaringClass().isEnum()
+        return ClassUtil.isEnumType(creator.getDeclaringClass())
                 && "valueOf".equals(creator.getName());
     }
-
-    /*
-    /**********************************************************
-    /* Helper class(es)
-    /**********************************************************
-     */
-
-    /**
-     * Replacement for default constructor to use for a small set of
-     * "well-known" types.
-     * <p>
-     * Note: replaces earlier <code>Vanilla</code>
-     * <code>ValueInstantiator</code> implementation
-     *
-     * @since 2.8.1 (replacing earlier <code>Vanilla</code> instantiator
-     */
-    protected final static class StdTypeConstructor extends AnnotatedWithParams
-            implements java.io.Serializable {
-        private static final long serialVersionUID = 1L;
-
-        public final static int TYPE_ARRAY_LIST = 1;
-        public final static int TYPE_HASH_MAP = 2;
-        public final static int TYPE_LINKED_HASH_MAP = 3;
-
-        private final AnnotatedWithParams _base;
-
-        private final int _type;
-
-        public StdTypeConstructor(AnnotatedWithParams base, int t) {
-            super(base, null);
-            _base = base;
-            _type = t;
-        }
-
-        public static AnnotatedWithParams tryToOptimize(
-                AnnotatedWithParams src) {
-            if (src != null) {
-                final Class<?> rawType = src.getDeclaringClass();
-                if (rawType == List.class || rawType == ArrayList.class) {
-                    return new StdTypeConstructor(src, TYPE_ARRAY_LIST);
-                }
-                if (rawType == LinkedHashMap.class) {
-                    return new StdTypeConstructor(src, TYPE_LINKED_HASH_MAP);
-                }
-                if (rawType == HashMap.class) {
-                    return new StdTypeConstructor(src, TYPE_HASH_MAP);
-                }
-            }
-            return src;
-        }
-
-        protected final Object _construct() {
-            switch (_type) {
-            case TYPE_ARRAY_LIST:
-                return new ArrayList<Object>();
-            case TYPE_LINKED_HASH_MAP:
-                return new LinkedHashMap<String, Object>();
-            case TYPE_HASH_MAP:
-                return new HashMap<String, Object>();
-            }
-            throw new IllegalStateException("Unknown type " + _type);
-        }
-
-        @Override
-        public int getParameterCount() {
-            return _base.getParameterCount();
-        }
-
-        @Override
-        public Class<?> getRawParameterType(int index) {
-            return _base.getRawParameterType(index);
-        }
-
-        @Override
-        public JavaType getParameterType(int index) {
-            return _base.getParameterType(index);
-        }
-
-        @Override
-        @Deprecated
-        public Type getGenericParameterType(int index) {
-            return _base.getGenericParameterType(index);
-        }
-
-        @Override
-        public Object call() throws Exception {
-            return _construct();
-        }
-
-        @Override
-        public Object call(Object[] args) throws Exception {
-            return _construct();
-        }
-
-        @Override
-        public Object call1(Object arg) throws Exception {
-            return _construct();
-        }
-
-        @Override
-        public Class<?> getDeclaringClass() {
-            return _base.getDeclaringClass();
-        }
-
-        @Override
-        public Member getMember() {
-            return _base.getMember();
-        }
-
-        @Override
-        public void setValue(Object pojo, Object value)
-                throws UnsupportedOperationException, IllegalArgumentException {
-            throw new UnsupportedOperationException();
-        }
-
-        @Override
-        public Object getValue(Object pojo)
-                throws UnsupportedOperationException, IllegalArgumentException {
-            throw new UnsupportedOperationException();
-        }
-
-        @Override
-        public Annotated withAnnotations(AnnotationMap fallback) {
-            throw new UnsupportedOperationException();
-        }
-
-        @Override
-        public AnnotatedElement getAnnotated() {
-            return _base.getAnnotated();
-        }
-
-        @Override
-        protected int getModifiers() {
-            return _base.getMember().getModifiers();
-        }
-
-        @Override
-        public String getName() {
-            return _base.getName();
-        }
-
-        @Override
-        public JavaType getType() {
-            return _base.getType();
-        }
-
-        @Override
-        public Class<?> getRawType() {
-            return _base.getRawType();
-        }
-
-        @Override
-        public boolean equals(Object o) {
-            return (o == this);
-        }
-
-        @Override
-        public int hashCode() {
-            return _base.hashCode();
-        }
-
-        @Override
-        public String toString() {
-            return _base.toString();
-        }
-    }
 }
