@@ -4,11 +4,11 @@ import java.io.IOException;
 import java.util.Collection;
 
 import com.fasterxml.jackson.core.JsonParser;
-import com.fasterxml.jackson.core.JsonProcessingException;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.deser.*;
 import com.fasterxml.jackson.databind.deser.impl.ObjectIdReader;
 import com.fasterxml.jackson.databind.jsontype.TypeDeserializer;
+import com.fasterxml.jackson.databind.util.AccessPattern;
 
 /**
  * Base class that simplifies implementations of {@link JsonDeserializer}s
@@ -32,22 +32,19 @@ public abstract class DelegatingDeserializer
     /**********************************************************************
      */
 
-    public DelegatingDeserializer(JsonDeserializer<?> delegatee)
+    public DelegatingDeserializer(JsonDeserializer<?> d)
     {
-        super(_figureType(delegatee));
-        _delegatee = delegatee;
+        super(d.handledType());
+        _delegatee = d;
     }
 
-    protected abstract JsonDeserializer<?> newDelegatingInstance(JsonDeserializer<?> newDelegatee);
+    /*
+    /**********************************************************************
+    /* Abstract methods to implement
+    /**********************************************************************
+     */
     
-    private static Class<?> _figureType(JsonDeserializer<?> deser)
-    {
-        Class<?> cls = deser.handledType();
-        if (cls != null) {
-            return cls;
-        }
-        return Object.class;
-    }
+    protected abstract JsonDeserializer<?> newDelegatingInstance(JsonDeserializer<?> newDelegatee);
     
     /*
     /**********************************************************************
@@ -67,68 +64,15 @@ public abstract class DelegatingDeserializer
             BeanProperty property)
         throws JsonMappingException
     {
-        JsonDeserializer<?> del = ctxt.handleSecondaryContextualization(_delegatee, property);
+        JavaType vt = ctxt.constructType(_delegatee.handledType());
+        JsonDeserializer<?> del = ctxt.handleSecondaryContextualization(_delegatee,
+                property, vt);
         if (del == _delegatee) {
             return this;
         }
         return newDelegatingInstance(del);
     }
 
-    /**
-     * @deprecated Since 2.3, use {@link #newDelegatingInstance} instead
-     */
-    @Deprecated
-    protected JsonDeserializer<?> _createContextual(DeserializationContext ctxt,
-            BeanProperty property, JsonDeserializer<?> newDelegatee)
-    {
-        if (newDelegatee == _delegatee) {
-            return this;
-        }
-        return newDelegatingInstance(newDelegatee);
-    }
-
-    @Override
-    public SettableBeanProperty findBackReference(String logicalName) {
-        // [Issue#253]: Hope this works....
-        return _delegatee.findBackReference(logicalName);
-    }
-
-    /*
-    /**********************************************************************
-    /* Overridden deserialization methods
-    /**********************************************************************
-     */
-
-    @Override
-    public Object deserialize(JsonParser jp, DeserializationContext ctxt)
-        throws IOException, JsonProcessingException
-    {
-        return _delegatee.deserialize(jp,  ctxt);
-    }
-
-    @SuppressWarnings("unchecked")
-    @Override
-    public Object deserialize(JsonParser jp, DeserializationContext ctxt,
-            Object intoValue)
-        throws IOException, JsonProcessingException
-    {
-        return ((JsonDeserializer<Object>)_delegatee).deserialize(jp, ctxt, intoValue);
-    }
-
-    @Override
-    public Object deserializeWithType(JsonParser jp, DeserializationContext ctxt,
-            TypeDeserializer typeDeserializer)
-        throws IOException, JsonProcessingException
-    {
-        return _delegatee.deserializeWithType(jp, ctxt, typeDeserializer);
-    }
-
-    /*
-    /**********************************************************************
-    /* Overridden other methods
-    /**********************************************************************
-     */
-
     @Override
     public JsonDeserializer<?> replaceDelegatee(JsonDeserializer<?> delegatee)
     {
@@ -138,23 +82,79 @@ public abstract class DelegatingDeserializer
         return newDelegatingInstance(delegatee);
     }
 
-    @Override
-    public Object getNullValue() { return _delegatee.getNullValue(); }
+    /*
+    /**********************************************************************
+    /* Overridden deserialization methods
+    /**********************************************************************
+     */
 
     @Override
-    public Object getEmptyValue() { return _delegatee.getEmptyValue(); }
+    public Object deserialize(JsonParser p, DeserializationContext ctxt)
+        throws IOException
+    {
+        return _delegatee.deserialize(p,  ctxt);
+    }
+
+    @SuppressWarnings("unchecked")
+    @Override
+    public Object deserialize(JsonParser p, DeserializationContext ctxt,
+            Object intoValue)
+        throws IOException
+    {
+        return ((JsonDeserializer<Object>)_delegatee).deserialize(p, ctxt, intoValue);
+    }
 
     @Override
-    public Collection<Object> getKnownPropertyNames() { return _delegatee.getKnownPropertyNames(); }
-    
+    public Object deserializeWithType(JsonParser p, DeserializationContext ctxt,
+            TypeDeserializer typeDeserializer)
+        throws IOException
+    {
+        return _delegatee.deserializeWithType(p, ctxt, typeDeserializer);
+    }
+
+    /*
+    /**********************************************************************
+    /* Overridden other methods
+    /**********************************************************************
+     */
+
     @Override
     public boolean isCachable() { return _delegatee.isCachable(); }
 
-    @Override
-    public ObjectIdReader getObjectIdReader() { return _delegatee.getObjectIdReader(); }
+    @Override // since 2.9
+    public Boolean supportsUpdate(DeserializationConfig config) {
+        return _delegatee.supportsUpdate(config);
+    }
 
     @Override
     public JsonDeserializer<?> getDelegatee() {
         return _delegatee;
     }
+
+    @Override
+    public SettableBeanProperty findBackReference(String logicalName) {
+        // [databind#253]: Hope this works....
+        return _delegatee.findBackReference(logicalName);
+    }
+
+    @Override
+    public AccessPattern getNullAccessPattern() {
+        return _delegatee.getNullAccessPattern();
+    }
+
+    @Override
+    public Object getNullValue(DeserializationContext ctxt) throws JsonMappingException {
+        return _delegatee.getNullValue(ctxt);
+    }
+
+    @Override
+    public Object getEmptyValue(DeserializationContext ctxt) throws JsonMappingException {
+        return _delegatee.getEmptyValue(ctxt);
+    }
+
+    @Override
+    public Collection<Object> getKnownPropertyNames() { return _delegatee.getKnownPropertyNames(); }
+
+    @Override
+    public ObjectIdReader getObjectIdReader() { return _delegatee.getObjectIdReader(); }
 }
