@@ -3,10 +3,12 @@ package com.fasterxml.jackson.databind.deser.std;
 import java.io.IOException;
 import java.util.Collection;
 
+import com.fasterxml.jackson.annotation.JsonFormat;
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JacksonStdImpl;
 import com.fasterxml.jackson.databind.deser.ContextualDeserializer;
+import com.fasterxml.jackson.databind.deser.NullValueProvider;
 import com.fasterxml.jackson.databind.deser.ValueInstantiator;
 import com.fasterxml.jackson.databind.introspect.AnnotatedWithParams;
 import com.fasterxml.jackson.databind.jsontype.TypeDeserializer;
@@ -25,8 +27,6 @@ public final class StringCollectionDeserializer
 
     // // Configuration
 
-    protected final JavaType _collectionType;
-    
     /**
      * Value deserializer to use, if NOT the standard one
      * (if it is, will be null).
@@ -57,31 +57,40 @@ public final class StringCollectionDeserializer
     public StringCollectionDeserializer(JavaType collectionType,
             JsonDeserializer<?> valueDeser, ValueInstantiator valueInstantiator)
     {
-        this(collectionType, valueInstantiator, null, valueDeser);
+        this(collectionType, valueInstantiator, null, valueDeser, valueDeser, null);
     }
 
     @SuppressWarnings("unchecked")
     protected StringCollectionDeserializer(JavaType collectionType,
             ValueInstantiator valueInstantiator, JsonDeserializer<?> delegateDeser,
-            JsonDeserializer<?> valueDeser)
+            JsonDeserializer<?> valueDeser,
+            NullValueProvider nuller, Boolean unwrapSingle)
     {
-        super(collectionType);
-        _collectionType = collectionType;
+        super(collectionType, nuller, unwrapSingle);
         _valueDeserializer = (JsonDeserializer<String>) valueDeser;
         _valueInstantiator = valueInstantiator;
         _delegateDeserializer = (JsonDeserializer<Object>) delegateDeser;
     }
 
     protected StringCollectionDeserializer withResolved(JsonDeserializer<?> delegateDeser,
-            JsonDeserializer<?> valueDeser)
+            JsonDeserializer<?> valueDeser,
+            NullValueProvider nuller, Boolean unwrapSingle)
     {
-        if ((_valueDeserializer == valueDeser) && (_delegateDeserializer == delegateDeser)) {
+        if ((_unwrapSingle == unwrapSingle) && (_nullProvider == nuller)
+                && (_valueDeserializer == valueDeser) && (_delegateDeserializer == delegateDeser)) {
             return this;
         }
-        return new StringCollectionDeserializer(_collectionType,
-                _valueInstantiator, delegateDeser, valueDeser);
+        return new StringCollectionDeserializer(_containerType, _valueInstantiator,
+                delegateDeser, valueDeser, nuller, unwrapSingle);
     }
 
+    @Override // since 2.5
+    public boolean isCachable() {
+        // 26-Mar-2015, tatu: Important: prevent caching if custom deserializers via annotations
+        //    are involved
+        return (_valueDeserializer == null) && (_delegateDeserializer == null);
+    }
+    
     /*
     /**********************************************************
     /* Validation, post-processing
@@ -94,27 +103,37 @@ public final class StringCollectionDeserializer
         // May need to resolve types for delegate-based creators:
         JsonDeserializer<Object> delegate = null;
         if (_valueInstantiator != null) {
-            AnnotatedWithParams delegateCreator = _valueInstantiator.getDelegateCreator();
+            // [databind#2324]: check both array-delegating and delegating
+            AnnotatedWithParams delegateCreator = _valueInstantiator.getArrayDelegateCreator();
             if (delegateCreator != null) {
+                JavaType delegateType = _valueInstantiator.getArrayDelegateType(ctxt.getConfig());
+                delegate = findDeserializer(ctxt, delegateType, property);
+            } else if ((delegateCreator = _valueInstantiator.getDelegateCreator()) != null) {
                 JavaType delegateType = _valueInstantiator.getDelegateType(ctxt.getConfig());
                 delegate = findDeserializer(ctxt, delegateType, property);
             }
         }
         JsonDeserializer<?> valueDeser = _valueDeserializer;
+        final JavaType valueType = _containerType.getContentType();
         if (valueDeser == null) {
-            // #125: May have a content converter
+            // [databind#125]: May have a content converter
             valueDeser = findConvertingContentDeserializer(ctxt, property, valueDeser);
             if (valueDeser == null) {
             // And we may also need to get deserializer for String
-                valueDeser = ctxt.findContextualValueDeserializer( _collectionType.getContentType(), property);
+                valueDeser = ctxt.findContextualValueDeserializer(valueType, property);
             }
         } else { // if directly assigned, probably not yet contextual, so:
-            valueDeser = ctxt.handleSecondaryContextualization(valueDeser, property);
-        } 
+            valueDeser = ctxt.handleSecondaryContextualization(valueDeser, property, valueType);
+        }
+        // 11-Dec-2015, tatu: Should we pass basic `Collection.class`, or more refined? Mostly
+        //   comes down to "List vs Collection" I suppose... for now, pass Collection
+        Boolean unwrapSingle = findFormatFeature(ctxt, property, Collection.class,
+                JsonFormat.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY);
+        NullValueProvider nuller = findContentNullProvider(ctxt, property, valueDeser);
         if (isDefaultDeserializer(valueDeser)) {
             valueDeser = null;
         }
-        return withResolved(delegate, valueDeser);
+        return withResolved(delegate, valueDeser, nuller, unwrapSingle);
     }
     
     /*
@@ -123,18 +142,18 @@ public final class StringCollectionDeserializer
     /**********************************************************
      */
 
-    @Override
-    public JavaType getContentType() {
-        return _collectionType.getContentType();
-    }
-
     @SuppressWarnings("unchecked")
     @Override
     public JsonDeserializer<Object> getContentDeserializer() {
         JsonDeserializer<?> deser = _valueDeserializer;
         return (JsonDeserializer<Object>) deser;
     }
-    
+
+    @Override
+    public ValueInstantiator getValueInstantiator() {
+        return _valueInstantiator;
+    }
+
     /*
     /**********************************************************
     /* JsonDeserializer API
@@ -143,41 +162,49 @@ public final class StringCollectionDeserializer
     
     @SuppressWarnings("unchecked")
     @Override
-    public Collection<String> deserialize(JsonParser jp, DeserializationContext ctxt)
+    public Collection<String> deserialize(JsonParser p, DeserializationContext ctxt)
         throws IOException
     {
         if (_delegateDeserializer != null) {
             return (Collection<String>) _valueInstantiator.createUsingDelegate(ctxt,
-                    _delegateDeserializer.deserialize(jp, ctxt));
+                    _delegateDeserializer.deserialize(p, ctxt));
         }
         final Collection<String> result = (Collection<String>) _valueInstantiator.createUsingDefault(ctxt);
-        return deserialize(jp, ctxt, result);
+        return deserialize(p, ctxt, result);
     }
 
     @Override
-    public Collection<String> deserialize(JsonParser jp, DeserializationContext ctxt,
+    public Collection<String> deserialize(JsonParser p, DeserializationContext ctxt,
             Collection<String> result)
         throws IOException
     {
         // Ok: must point to START_ARRAY
-        if (!jp.isExpectedStartArrayToken()) {
-            return handleNonArray(jp, ctxt, result);
+        if (!p.isExpectedStartArrayToken()) {
+            return handleNonArray(p, ctxt, result);
         }
 
         if (_valueDeserializer != null) {
-            return deserializeUsingCustom(jp, ctxt, result, _valueDeserializer);
+            return deserializeUsingCustom(p, ctxt, result, _valueDeserializer);
         }
-        JsonToken t;
-
         try {
-            while ((t = jp.nextToken()) != JsonToken.END_ARRAY) {
-                String value;
-                if (t == JsonToken.VALUE_STRING) {
-                    value = jp.getText();
-                } else if (t == JsonToken.VALUE_NULL) {
-                    value = null;
+            while (true) {
+                // First the common case:
+                String value = p.nextTextValue();
+                if (value != null) {
+                    result.add(value);
+                    continue;
+                }
+                JsonToken t = p.getCurrentToken();
+                if (t == JsonToken.END_ARRAY) {
+                    break;
+                }
+                if (t == JsonToken.VALUE_NULL) {
+                    if (_skipNullValues) {
+                        continue;
+                    }
+                    value = (String) _nullProvider.getNullValue(ctxt);
                 } else {
-                    value = _parseString(jp, ctxt);
+                    value = _parseString(p, ctxt);
                 }
                 result.add(value);
             }
@@ -187,50 +214,83 @@ public final class StringCollectionDeserializer
         return result;
     }
     
-    private Collection<String> deserializeUsingCustom(JsonParser jp, DeserializationContext ctxt,
+    private Collection<String> deserializeUsingCustom(JsonParser p, DeserializationContext ctxt,
             Collection<String> result, final JsonDeserializer<String> deser) throws IOException
     {
-        JsonToken t;
-        while ((t = jp.nextToken()) != JsonToken.END_ARRAY) {
-            String value;
-
-            if (t == JsonToken.VALUE_NULL) {
-                value = deser.getNullValue();
-            } else {
-                value = deser.deserialize(jp, ctxt);
+        try {
+            while (true) {
+                /* 30-Dec-2014, tatu: This may look odd, but let's actually call method
+                 *   that suggest we are expecting a String; this helps with some formats,
+                 *   notably XML. Note, however, that while we can get String, we can't
+                 *   assume that's what we use due to custom deserializer
+                 */
+                String value;
+                if (p.nextTextValue() == null) {
+                    JsonToken t = p.getCurrentToken();
+                    if (t == JsonToken.END_ARRAY) {
+                        break;
+                    }
+                    // Ok: no need to convert Strings, but must recognize nulls
+                    if (t == JsonToken.VALUE_NULL) {
+                        if (_skipNullValues) {
+                            continue;
+                        }
+                        value = (String) _nullProvider.getNullValue(ctxt);
+                    } else {
+                        value = deser.deserialize(p, ctxt);
+                    }
+                } else {
+                    value = deser.deserialize(p, ctxt);
+                }
+                result.add(value);
             }
-            result.add(value);
+        } catch (Exception e) {
+            throw JsonMappingException.wrapWithPath(e, result, result.size());
         }
         return result;
     }
-    
+
     @Override
-    public Object deserializeWithType(JsonParser jp, DeserializationContext ctxt, TypeDeserializer typeDeserializer) throws IOException {
+    public Object deserializeWithType(JsonParser p, DeserializationContext ctxt,
+            TypeDeserializer typeDeserializer) throws IOException {
         // In future could check current token... for now this should be enough:
-        return typeDeserializer.deserializeTypedFromArray(jp, ctxt);
+        return typeDeserializer.deserializeTypedFromArray(p, ctxt);
     }
 
     /**
-     * Helper method called when current token is no START_ARRAY. Will either
+     * Helper method called when current token is not START_ARRAY. Will either
      * throw an exception, or try to handle value as if member of implicit
      * array, depending on configuration.
      */
-    private final Collection<String> handleNonArray(JsonParser jp, DeserializationContext ctxt, Collection<String> result) throws IOException
+    @SuppressWarnings("unchecked")
+    private final Collection<String> handleNonArray(JsonParser p, DeserializationContext ctxt,
+            Collection<String> result) throws IOException
     {
-        // [JACKSON-526]: implicit arrays from single values?
-        if (!ctxt.isEnabled(DeserializationFeature.ACCEPT_SINGLE_VALUE_AS_ARRAY)) {
-            throw ctxt.mappingException(_collectionType.getRawClass());
+        // implicit arrays from single values?
+        boolean canWrap = (_unwrapSingle == Boolean.TRUE) ||
+                ((_unwrapSingle == null) &&
+                        ctxt.isEnabled(DeserializationFeature.ACCEPT_SINGLE_VALUE_AS_ARRAY));
+        if (!canWrap) {
+            return (Collection<String>) ctxt.handleUnexpectedToken(_containerType.getRawClass(), p);
         }
         // Strings are one of "native" (intrinsic) types, so there's never type deserializer involved
         JsonDeserializer<String> valueDes = _valueDeserializer;
-        JsonToken t = jp.getCurrentToken();
+        JsonToken t = p.getCurrentToken();
 
         String value;
         
         if (t == JsonToken.VALUE_NULL) {
-            value = (valueDes == null) ? null : valueDes.getNullValue();
+            // 03-Feb-2017, tatu: Does this work?
+            if (_skipNullValues) {
+                return result;
+            }
+            value = (String) _nullProvider.getNullValue(ctxt);
         } else {
-            value = (valueDes == null) ? _parseString(jp, ctxt) : valueDes.deserialize(jp, ctxt);
+            try {
+                value = (valueDes == null) ? _parseString(p, ctxt) : valueDes.deserialize(p, ctxt);
+            } catch (Exception e) {
+                throw JsonMappingException.wrapWithPath(e, result, result.size());
+            }
         }
         result.add(value);
         return result;
