@@ -4,8 +4,10 @@ import java.lang.reflect.Constructor;
 import java.lang.reflect.Method;
 import java.util.*;
 
+import com.fasterxml.jackson.annotation.JsonCreator;
 import com.fasterxml.jackson.annotation.JsonFormat;
 import com.fasterxml.jackson.annotation.JsonInclude;
+
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JsonPOJOBuilder;
 import com.fasterxml.jackson.databind.cfg.HandlerInstantiator;
@@ -16,32 +18,55 @@ import com.fasterxml.jackson.databind.util.ClassUtil;
 import com.fasterxml.jackson.databind.util.Converter;
 
 /**
- * Default {@link BeanDescription} implementation.
- * Can theoretically be subclassed to customize
- * some aspects of property introspection.
+ * Default {@link BeanDescription} implementation used by Jackson.
+ *<p>
+ * Although sub-classing is a theoretical possibility there are no known
+ * use cases for that, nor is such usage tested or supported.
+ * Separation from API is mostly to isolate some implementation details
+ * here and keep API simple.
  */
 public class BasicBeanDescription extends BeanDescription
 {
+    // since 2.9
+    private final static Class<?>[] NO_VIEWS = new Class<?>[0];
+
     /*
     /**********************************************************
     /* General configuration
     /**********************************************************
      */
 
+    /**
+     * We will hold a reference to the collector in cases where
+     * information is lazily accessed and constructed; properties
+     * are only accessed when they are actually needed.
+     */
+    final protected POJOPropertiesCollector _propCollector;
+    
     final protected MapperConfig<?> _config;
 
     final protected AnnotationIntrospector _annotationIntrospector;
+
+    /*
+    /**********************************************************
+    /* Information about type itself
+    /**********************************************************
+     */
     
     /**
      * Information collected about the class introspected.
      */
     final protected AnnotatedClass _classInfo;
-    
+
     /**
-     * We may need type bindings for the bean type. If so, we'll
-     * construct it lazily
+     * @since 2.9
      */
-    protected TypeBindings _bindings;
+    protected Class<?>[] _defaultViews;
+
+    /**
+     * @since 2.9
+     */
+    protected boolean _defaultViewsResolved;
 
     /*
     /**********************************************************
@@ -50,53 +75,59 @@ public class BasicBeanDescription extends BeanDescription
      */
 
     /**
-     * Properties collected for the POJO.
+     * Properties collected for the POJO; initialized as needed.
      */
-    protected final List<BeanPropertyDefinition> _properties;
+    protected List<BeanPropertyDefinition> _properties;
 
     /**
      * Details of Object Id to include, if any
      */
     protected ObjectIdInfo _objectIdInfo;
-    
-    // // for deserialization
-    
-    protected AnnotatedMethod _anySetterMethod;
 
-    protected Map<Object, AnnotatedMember> _injectables;
-    
-    /**
-     * Set of properties that can be ignored during deserialization, due
-     * to being marked as ignored.
-     */
-    protected Set<String> _ignoredPropertyNames;
-
-    // // for serialization
-    
-    protected AnnotatedMethod _jsonValueMethod;
-
-    protected AnnotatedMember _anyGetter;
-    
     /*
     /**********************************************************
     /* Life-cycle
     /**********************************************************
      */
 
-    protected BasicBeanDescription(MapperConfig<?> config,
-            JavaType type, AnnotatedClass classDef,
-            List<BeanPropertyDefinition> props)
+    protected BasicBeanDescription(POJOPropertiesCollector coll,
+            JavaType type, AnnotatedClass classDef)
     {
         super(type);
+        _propCollector = coll;
+        _config = coll.getConfig();
+        // NOTE: null config only for some pre-constructed types
+        if (_config == null) {
+            _annotationIntrospector = null;
+        } else {
+            _annotationIntrospector = _config.getAnnotationIntrospector();
+        }
+        _classInfo = classDef;
+    }
+
+    /**
+     * Alternate constructor used in cases where property information is not needed,
+     * only class info.
+     */
+    protected BasicBeanDescription(MapperConfig<?> config,
+            JavaType type, AnnotatedClass classDef, List<BeanPropertyDefinition> props)
+    {
+        super(type);
+        _propCollector = null;
         _config = config;
-        _annotationIntrospector = (config == null) ? null : config.getAnnotationIntrospector();
+        // NOTE: null config only for some pre-constructed types
+        if (_config == null) {
+            _annotationIntrospector = null;
+        } else {
+            _annotationIntrospector = _config.getAnnotationIntrospector();
+        }
         _classInfo = classDef;
         _properties = props;
     }
     
     protected BasicBeanDescription(POJOPropertiesCollector coll)
     {
-        this(coll.getConfig(), coll.getType(), coll.getClassDef(), coll.getProperties());
+        this(coll, coll.getType(), coll.getClassDef());
         _objectIdInfo = coll.getObjectIdInfo();
     }
 
@@ -104,26 +135,16 @@ public class BasicBeanDescription extends BeanDescription
      * Factory method to use for constructing an instance to use for building
      * deserializers.
      */
-    public static BasicBeanDescription forDeserialization(POJOPropertiesCollector coll)
-    {
-        BasicBeanDescription desc = new BasicBeanDescription(coll);
-        desc._anySetterMethod = coll.getAnySetterMethod();
-        desc._ignoredPropertyNames = coll.getIgnoredPropertyNames();
-        desc._injectables = coll.getInjectables();
-        desc._jsonValueMethod = coll.getJsonValueMethod();
-        return desc;
+    public static BasicBeanDescription forDeserialization(POJOPropertiesCollector coll) {
+        return new BasicBeanDescription(coll);
     }
 
     /**
      * Factory method to use for constructing an instance to use for building
      * serializers.
      */
-    public static BasicBeanDescription forSerialization(POJOPropertiesCollector coll)
-    {
-        BasicBeanDescription desc = new BasicBeanDescription(coll);
-        desc._jsonValueMethod = coll.getJsonValueMethod();
-        desc._anyGetter = coll.getAnyGetter();
-        return desc;
+    public static BasicBeanDescription forSerialization(POJOPropertiesCollector coll) {
+        return new BasicBeanDescription(coll);
     }
 
     /**
@@ -138,6 +159,13 @@ public class BasicBeanDescription extends BeanDescription
                 ac, Collections.<BeanPropertyDefinition>emptyList());
     }
 
+    protected List<BeanPropertyDefinition> _properties() {
+        if (_properties == null) {
+            _properties = _propCollector.getProperties();
+        }
+        return _properties;
+    }
+
     /*
     /**********************************************************
     /* Limited modifications by core databind functionality
@@ -153,7 +181,7 @@ public class BasicBeanDescription extends BeanDescription
      */
     public boolean removeProperty(String propName)
     {
-        Iterator<BeanPropertyDefinition> it = _properties.iterator();
+        Iterator<BeanPropertyDefinition> it = _properties().iterator();
         while (it.hasNext()) {
             BeanPropertyDefinition prop = it.next();
             if (prop.getName().equals(propName)) {
@@ -163,6 +191,36 @@ public class BasicBeanDescription extends BeanDescription
         }
         return false;
     }
+
+    public boolean addProperty(BeanPropertyDefinition def)
+    {
+        // first: ensure we do not have such property
+        if (hasProperty(def.getFullName())) {
+            return false;
+        }
+        _properties().add(def);
+        return true;
+    }
+    
+    /**
+     * @since 2.6
+     */
+    public boolean hasProperty(PropertyName name) {
+        return findProperty(name) != null;
+    }
+    
+    /**
+     * @since 2.6
+     */
+    public BeanPropertyDefinition findProperty(PropertyName name)
+    {
+        for (BeanPropertyDefinition prop : _properties()) {
+            if (prop.hasName(name)) {
+                return prop;
+            }
+        }
+        return null;
+    }
     
     /*
     /**********************************************************
@@ -175,25 +233,35 @@ public class BasicBeanDescription extends BeanDescription
 
     @Override
     public ObjectIdInfo getObjectIdInfo() { return  _objectIdInfo; }
-    
+
     @Override
     public List<BeanPropertyDefinition> findProperties() {
-        return _properties;
+        return _properties();
     }
 
     @Override
+    @Deprecated // since 2.9
     public AnnotatedMethod findJsonValueMethod() {
-        return _jsonValueMethod;
+        return (_propCollector == null) ? null
+                : _propCollector.getJsonValueMethod();
     }
 
+    @Override // since 2.9
+    public AnnotatedMember findJsonValueAccessor() {
+        return (_propCollector == null) ? null
+                : _propCollector.getJsonValueAccessor();
+    }
+ 
     @Override
     public Set<String> getIgnoredPropertyNames() {
-        if (_ignoredPropertyNames == null) {
+        Set<String> ign = (_propCollector == null) ? null
+                : _propCollector.getIgnoredPropertyNames();
+        if (ign == null) {
             return Collections.emptySet();
         }
-        return _ignoredPropertyNames;
+        return ign;
     }
-    
+
     @Override
     public boolean hasKnownClassAnnotations() {
         return _classInfo.hasAnnotations();
@@ -205,20 +273,18 @@ public class BasicBeanDescription extends BeanDescription
     }
 
     @Override
-    public TypeBindings bindingsForBeanType()
-    {
-        if (_bindings == null) {
-            _bindings = new TypeBindings(_config.getTypeFactory(), _type);
-        }
-        return _bindings;
+    @Deprecated // since 2.7
+    public TypeBindings bindingsForBeanType() {
+        return _type.getBindings();
     }
 
     @Override
+    @Deprecated // since 2.8
     public JavaType resolveType(java.lang.reflect.Type jdkType) {
         if (jdkType == null) {
             return null;
         }
-        return bindingsForBeanType().resolveType(jdkType);
+        return _config.getTypeFactory().constructType(jdkType, _type.getBindings());
     }
 
     @Override
@@ -227,28 +293,47 @@ public class BasicBeanDescription extends BeanDescription
     }
 
     @Override
-    public AnnotatedMethod findAnySetter() throws IllegalArgumentException
+    public AnnotatedMember findAnySetterAccessor() throws IllegalArgumentException
     {
-        if (_anySetterMethod != null) {
-            /* Also, let's be somewhat strict on how field name is to be
-             * passed; String, Object make sense, others not
-             * so much.
-             */
-            /* !!! 18-May-2009, tatu: how about enums? Can add support if
-             *  requested; easy enough for devs to add support within
-             *  method.
-             */
-            Class<?> type = _anySetterMethod.getRawParameterType(0);
-            if (type != String.class && type != Object.class) {
-                throw new IllegalArgumentException("Invalid 'any-setter' annotation on method "+_anySetterMethod.getName()+"(): first argument not of type String or Object, but "+type.getName());
+        if (_propCollector != null) {
+            AnnotatedMethod anyMethod = _propCollector.getAnySetterMethod();
+            if (anyMethod != null) {
+                // Also, let's be somewhat strict on how field name is to be
+                // passed; String, Object make sense, others not so much.
+    
+                /* !!! 18-May-2009, tatu: how about enums? Can add support if
+                 *  requested; easy enough for devs to add support within method.
+                 */
+                Class<?> type = anyMethod.getRawParameterType(0);
+                if ((type != String.class) && (type != Object.class)) {
+                    throw new IllegalArgumentException(String.format(
+"Invalid 'any-setter' annotation on method '%s()': first argument not of type String or Object, but %s",
+anyMethod.getName(), type.getName()));
+                }
+                return anyMethod;
+            }
+            AnnotatedMember anyField = _propCollector.getAnySetterField();
+            if (anyField != null) {
+                // For now let's require a Map; in future can add support for other
+                // types like perhaps Iterable<Map.Entry>?
+                Class<?> type = anyField.getRawType();
+                if (!Map.class.isAssignableFrom(type)) {
+                    throw new IllegalArgumentException(String.format(
+"Invalid 'any-setter' annotation on field '%s': type is not instance of java.util.Map",
+anyField.getName()));
+                }
+                return anyField;
             }
         }
-        return _anySetterMethod;
+        return null;
     }
 
     @Override
     public Map<Object, AnnotatedMember> findInjectables() {
-        return _injectables;
+        if (_propCollector != null) {
+            return _propCollector.getInjectables();
+        }
+        return Collections.emptyMap();
     }
 
     @Override
@@ -257,14 +342,13 @@ public class BasicBeanDescription extends BeanDescription
     }
 
     @Override
-    public Object instantiateBean(boolean fixAccess)
-    {
+    public Object instantiateBean(boolean fixAccess) {
         AnnotatedConstructor ac = _classInfo.getDefaultConstructor();
         if (ac == null) {
             return null;
         }
         if (fixAccess) {
-            ac.fixAccess();
+            ac.fixAccess(_config.isEnabled(MapperFeature.OVERRIDE_PUBLIC_ACCESS_MODIFIERS));
         }
         try {
             return ac.getAnnotated().newInstance();
@@ -273,12 +357,14 @@ public class BasicBeanDescription extends BeanDescription
             while (t.getCause() != null) {
                 t = t.getCause();
             }
-            if (t instanceof Error) throw (Error) t;
-            if (t instanceof RuntimeException) throw (RuntimeException) t;
-            throw new IllegalArgumentException("Failed to instantiate bean of type "+_classInfo.getAnnotated().getName()+": ("+t.getClass().getName()+") "+t.getMessage(), t);
+            ClassUtil.throwIfError(t);
+            ClassUtil.throwIfRTE(t);
+            throw new IllegalArgumentException("Failed to instantiate bean of type "
+                    +_classInfo.getAnnotated().getName()+": ("+t.getClass().getName()+") "
+                    +ClassUtil.exceptionMessage(t), t);
         }
     }
-    
+
     /*
     /**********************************************************
     /* Simple accessors, extended
@@ -299,15 +385,47 @@ public class BasicBeanDescription extends BeanDescription
     @Override
     public JsonFormat.Value findExpectedFormat(JsonFormat.Value defValue)
     {
+        // 15-Apr-2016, tatu: Let's check both per-type defaults and annotations; per-type
+        //   defaults having higher precedence, so start with that
         if (_annotationIntrospector != null) {
             JsonFormat.Value v = _annotationIntrospector.findFormat(_classInfo);
             if (v != null) {
-                return v;
+                if (defValue == null) {
+                    defValue = v;
+                } else {
+                    defValue = defValue.withOverrides(v);
+                }
+            }
+        }
+        JsonFormat.Value v = _config.getDefaultPropertyFormat(_classInfo.getRawType());
+        if (v != null) {
+            if (defValue == null) {
+                defValue = v;
+            } else {
+                defValue = defValue.withOverrides(v);
             }
         }
         return defValue;
     }
-    
+
+    @Override // since 2.9
+    public Class<?>[] findDefaultViews()
+    {
+        if (!_defaultViewsResolved) {
+            _defaultViewsResolved = true;
+            Class<?>[] def = (_annotationIntrospector == null) ? null
+                    : _annotationIntrospector.findViews(_classInfo);
+            // one more twist: if default inclusion disabled, need to force empty set of views
+            if (def == null) {
+                if (!_config.isEnabled(MapperFeature.DEFAULT_VIEW_INCLUSION)) {
+                    def = NO_VIEWS;
+                }
+            }
+            _defaultViews = def;
+        }
+        return _defaultViews;
+    }
+
     /*
     /**********************************************************
     /* Introspection for serialization
@@ -330,14 +448,16 @@ public class BasicBeanDescription extends BeanDescription
      * and per-class annotation (highest priority).
      */
     @Override
-    public JsonInclude.Include findSerializationInclusion(JsonInclude.Include defValue)
-    {
-        if (_annotationIntrospector == null) {
-            return defValue;
+    public JsonInclude.Value findPropertyInclusion(JsonInclude.Value defValue) {
+        if (_annotationIntrospector != null) {
+            JsonInclude.Value incl = _annotationIntrospector.findPropertyInclusion(_classInfo);
+            if (incl != null) {
+                return (defValue == null) ? incl : defValue.withOverrides(incl);
+            }
         }
-        return _annotationIntrospector.findSerializationInclusion(_classInfo, defValue);
+        return defValue;
     }
-    
+
     /**
      * Method used to locate the method of introspected class that
      * implements {@link com.fasterxml.jackson.annotation.JsonAnyGetter}.
@@ -347,37 +467,56 @@ public class BasicBeanDescription extends BeanDescription
     @Override
     public AnnotatedMember findAnyGetter() throws IllegalArgumentException
     {
-        if (_anyGetter != null) {
+        AnnotatedMember anyGetter = (_propCollector == null) ? null
+                : _propCollector.getAnyGetter();
+        if (anyGetter != null) {
             /* For now let's require a Map; in future can add support for other
              * types like perhaps Iterable<Map.Entry>?
              */
-            Class<?> type = _anyGetter.getRawType();
+            Class<?> type = anyGetter.getRawType();
             if (!Map.class.isAssignableFrom(type)) {
-                throw new IllegalArgumentException("Invalid 'any-getter' annotation on method "+_anyGetter.getName()+"(): return type is not instance of java.util.Map");
+                throw new IllegalArgumentException("Invalid 'any-getter' annotation on method "+anyGetter.getName()+"(): return type is not instance of java.util.Map");
             }
         }
-        return _anyGetter;
+        return anyGetter;
     }
-    
+
     @Override
-    public Map<String,AnnotatedMember> findBackReferenceProperties()
+    public List<BeanPropertyDefinition> findBackReferences()
     {
-        HashMap<String,AnnotatedMember> result = null;
-        for (BeanPropertyDefinition property : _properties) {
-            AnnotatedMember am = property.getMutator();
-            if (am == null) {
+        List<BeanPropertyDefinition> result = null;
+        HashSet<String> names = null;
+        for (BeanPropertyDefinition property : _properties()) {
+            AnnotationIntrospector.ReferenceProperty refDef = property.findReferenceType();
+            if ((refDef == null) || !refDef.isBackReference()) {
                 continue;
             }
-            AnnotationIntrospector.ReferenceProperty refDef = _annotationIntrospector.findReferenceType(am);
-            if (refDef != null && refDef.isBackReference()) {
-                if (result == null) {
-                    result = new HashMap<String,AnnotatedMember>();
-                }
-                String refName = refDef.getName();
-                if (result.put(refName, am) != null) {
+            final String refName = refDef.getName();
+            if (result == null) {
+                result = new ArrayList<BeanPropertyDefinition>();
+                names = new HashSet<>();
+                names.add(refName);
+            } else {
+                if (!names.add(refName)) {
                     throw new IllegalArgumentException("Multiple back-reference properties with name '"+refName+"'");
                 }
             }
+            result.add(property);
+        }
+        return result;
+    }
+
+    @Deprecated // since 2.9
+    @Override
+    public Map<String,AnnotatedMember> findBackReferenceProperties()
+    {
+        List<BeanPropertyDefinition> props = findBackReferences();
+        if (props == null) {
+            return null;
+        }
+        Map<String,AnnotatedMember> result = new HashMap<>();
+        for (BeanPropertyDefinition prop : props) {
+            result.put(prop.getName(), prop.getMutator());
         }
         return result;
     }
@@ -392,16 +531,22 @@ public class BasicBeanDescription extends BeanDescription
     public List<AnnotatedMethod> getFactoryMethods()
     {
         // must filter out anything that clearly is not a factory method
-        List<AnnotatedMethod> candidates = _classInfo.getStaticMethods();
+        List<AnnotatedMethod> candidates = _classInfo.getFactoryMethods();
         if (candidates.isEmpty()) {
             return candidates;
         }
-        ArrayList<AnnotatedMethod> result = new ArrayList<AnnotatedMethod>();
+        List<AnnotatedMethod> result = null;
         for (AnnotatedMethod am : candidates) {
             if (isFactoryMethod(am)) {
+                if (result == null) {
+                    result = new ArrayList<AnnotatedMethod>();
+                }
                 result.add(am);
             }
         }
+        if (result == null) {
+            return Collections.emptyList();
+        }
         return result;
     }
 
@@ -429,8 +574,9 @@ public class BasicBeanDescription extends BeanDescription
     public Method findFactoryMethod(Class<?>... expArgTypes)
     {
         // So, of all single-arg static methods:
-        for (AnnotatedMethod am : _classInfo.getStaticMethods()) {
-            if (isFactoryMethod(am)) {
+        for (AnnotatedMethod am : _classInfo.getFactoryMethods()) {
+            // 24-Oct-2016, tatu: Better ensure it only takes 1 arg, no matter what
+            if (isFactoryMethod(am) && am.getParameterCount() == 1) {
                 // And must take one of expected arg types (or supertype)
                 Class<?> actualArgType = am.getRawParameterType(0);
                 for (Class<?> expArgType : expArgTypes) {
@@ -446,29 +592,30 @@ public class BasicBeanDescription extends BeanDescription
 
     protected boolean isFactoryMethod(AnnotatedMethod am)
     {
-        /* First: return type must be compatible with the introspected class
-         * (i.e. allowed to be sub-class, although usually is the same
-         * class)
-         */
+        // First: return type must be compatible with the introspected class
+        // (i.e. allowed to be sub-class, although usually is the same class)
         Class<?> rt = am.getRawReturnType();
         if (!getBeanClass().isAssignableFrom(rt)) {
             return false;
         }
-
         /* Also: must be a recognized factory method, meaning:
          * (a) marked with @JsonCreator annotation, or
-         * (a) "valueOf" (at this point, need not be public)
+         * (b) "valueOf" (at this point, need not be public)
          */
-        if (_annotationIntrospector.hasCreatorAnnotation(am)) {
+        JsonCreator.Mode mode = _annotationIntrospector.findCreatorAnnotation(_config, am);
+        if ((mode != null) && (mode != JsonCreator.Mode.DISABLED)) {
             return true;
         }
         final String name = am.getName();
+        // 24-Oct-2016, tatu: As per [databind#1429] must ensure takes exactly one arg
         if ("valueOf".equals(name)) {
-            return true;
+            if (am.getParameterCount() == 1) {
+                return true;
+            }
         }
-        // [Issue#208] Also accept "fromString()", if takes String or CharSequence
+        // [databind#208] Also accept "fromString()", if takes String or CharSequence
         if ("fromString".equals(name)) {
-            if (1 == am.getParameterCount()) {
+            if (am.getParameterCount() == 1) {
                 Class<?> cls = am.getRawParameterType(0);
                 if (cls == String.class || CharSequence.class.isAssignableFrom(cls)) {
                     return true;
@@ -479,79 +626,30 @@ public class BasicBeanDescription extends BeanDescription
     }
 
     /**
-     * @deprecated Since 2.4, use {@link #findCreatorParameterNames()} instead.
+     * @deprecated since 2.8
      */
-    @Deprecated
-    public List<String> findCreatorPropertyNames()
-    {
-        List<PropertyName> params = findCreatorParameterNames();
-        if (params.isEmpty()) {
-    		    return Collections.emptyList();
-        }
-        List<String> result = new ArrayList<String>(params.size());
-        for (PropertyName name : params) {
-    		    result.add(name.getSimpleName());
-        }
-        return result;
-    }
-    
-    /**
-     * Method for getting ordered list of named Creator properties.
-     * Returns an empty list is none found. If multiple Creator
-     * methods are defined, order between properties from different
-     * methods is undefined; however, properties for each such
-     * Creator are ordered properly relative to each other.
-     * For the usual case of just a single Creator, named properties are
-     * thus properly ordered.
-     * 
-     * @since 2.4
-     */
-    public List<PropertyName> findCreatorParameterNames()
-    {
-        for (int i = 0; i < 2; ++i) {
-            List<? extends AnnotatedWithParams> l = (i == 0)
-                ? getConstructors() : getFactoryMethods();
-            for (AnnotatedWithParams creator : l) {
-                int argCount = creator.getParameterCount();
-                if (argCount < 1) continue;
-                PropertyName name = _findCreatorPropertyName(creator.getParameter(0));
-                if (name == null || name.isEmpty()) {
-                    continue;
-                }
-                List<PropertyName> names = new ArrayList<PropertyName>();
-                names.add(name);
-                for (int p = 1; p < argCount; ++p) {
-                    name = _findCreatorPropertyName(creator.getParameter(p));
-                    names.add(name);
-                }
-                return names;
-            }
-        }
-        return Collections.emptyList();
-    }
-
+    @Deprecated // since 2.8, not used at least since 2.7
     protected PropertyName _findCreatorPropertyName(AnnotatedParameter param)
     {
-    	PropertyName name = _annotationIntrospector.findNameForDeserialization(param);
-    	if (name == null || name.isEmpty()) {
-    		String str = _annotationIntrospector.findImplicitPropertyName(param);
-    		if (str != null && !str.isEmpty()) {
-    			name = new PropertyName(str);
-    		}
-    	}
-    	return name;
+        PropertyName name = _annotationIntrospector.findNameForDeserialization(param);
+        if (name == null || name.isEmpty()) {
+            String str = _annotationIntrospector.findImplicitPropertyName(param);
+            if (str != null && !str.isEmpty()) {
+                name = PropertyName.construct(str);
+            }
+        }
+        return name;
     }
-    
+
     /*
     /**********************************************************
     /* Introspection for deserialization, other
     /**********************************************************
      */
-    
+
     @Override
-    public Class<?> findPOJOBuilder()
-    {
-    	return (_annotationIntrospector == null) ?
+    public Class<?> findPOJOBuilder() {
+        return (_annotationIntrospector == null) ?
     			null : _annotationIntrospector.findPOJOBuilder(_classInfo);
     }
 
@@ -571,6 +669,12 @@ public class BasicBeanDescription extends BeanDescription
         return _createConverter(_annotationIntrospector.findDeserializationConverter(_classInfo));
     }
 
+    @Override
+    public String findClassDescription() {
+        return (_annotationIntrospector == null) ?
+                null : _annotationIntrospector.findClassDescription(_classInfo);
+    }
+
     /*
     /**********************************************************
     /* Helper methods for field introspection
@@ -586,12 +690,15 @@ public class BasicBeanDescription extends BeanDescription
      *
      * @return Ordered Map with logical property name as key, and
      *    matching field as value.
+     *
+     * @deprecated Since 2.7.2, does not seem to be used?
      */
+    @Deprecated
     public LinkedHashMap<String,AnnotatedField> _findPropertyFields(
             Collection<String> ignoredProperties, boolean forSerialization)
     {
         LinkedHashMap<String,AnnotatedField> results = new LinkedHashMap<String,AnnotatedField>();
-        for (BeanPropertyDefinition property : _properties) {
+        for (BeanPropertyDefinition property : _properties()) {
             AnnotatedField f = property.getField();
             if (f != null) {
                 String name = property.getName();
@@ -613,7 +720,7 @@ public class BasicBeanDescription extends BeanDescription
      */
     
     @SuppressWarnings("unchecked")
-    public Converter<Object,Object> _createConverter(Object converterDef)
+    protected Converter<Object,Object> _createConverter(Object converterDef)
     {
         if (converterDef == null) {
             return null;
