@@ -66,7 +66,12 @@ public class BasicClassIntrospector
         // a small cache should go a long way here
         _cachedFCA = new LRUMap<JavaType,BasicBeanDescription>(16, 64);
     }
-    
+
+    @Override
+    public ClassIntrospector copy() {
+        return new BasicClassIntrospector();
+    }
+
     /*
     /**********************************************************
     /* Factory method impls
@@ -132,8 +137,7 @@ public class BasicClassIntrospector
     {
         BasicBeanDescription desc = _findStdTypeDesc(type);
         if (desc == null) {
-
-            // As per [Databind#550], skip full introspection for some of standard
+            // As per [databind#550], skip full introspection for some of standard
             // structured types as well
             desc = _findStdJdkCollectionDesc(cfg, type);
             if (desc == null) {
