@@ -997,15 +997,21 @@ public class JacksonAnnotationIntrospector
     @Override
     public PropertyName findNameForSerialization(Annotated a)
     {
+        boolean useDefault = false;
         JsonGetter jg = _findAnnotation(a, JsonGetter.class);
         if (jg != null) {
-            return PropertyName.construct(jg.value());
+            String s = jg.value();
+            // 04-May-2018, tatu: Should allow for "nameless" `@JsonGetter` too
+            if (!s.isEmpty()) {
+                return PropertyName.construct(s);
+            }
+            useDefault = true;
         }
         JsonProperty pann = _findAnnotation(a, JsonProperty.class);
         if (pann != null) {
             return PropertyName.construct(pann.value());
         }
-        if (_hasOneOf(a, ANNOTATIONS_TO_INFER_SER)) {
+        if (useDefault || _hasOneOf(a, ANNOTATIONS_TO_INFER_SER)) {
             return PropertyName.USE_DEFAULT;
         }
         return null;
@@ -1228,16 +1234,23 @@ public class JacksonAnnotationIntrospector
     public PropertyName findNameForDeserialization(Annotated a)
     {
         // @JsonSetter has precedence over @JsonProperty, being more specific
-        // @JsonDeserialize implies that there is a property, but no name
+
+        boolean useDefault = false;
         JsonSetter js = _findAnnotation(a, JsonSetter.class);
         if (js != null) {
-            return PropertyName.construct(js.value());
+            String s = js.value();
+            // 04-May-2018, tatu: Need to allow for "nameless" `@JsonSetter` too
+            if (s.isEmpty()) {
+                useDefault = true;
+            } else {
+                return PropertyName.construct(s);
+            }
         }
         JsonProperty pann = _findAnnotation(a, JsonProperty.class);
         if (pann != null) {
             return PropertyName.construct(pann.value());
         }
-        if (_hasOneOf(a, ANNOTATIONS_TO_INFER_DESER)) {
+        if (useDefault || _hasOneOf(a, ANNOTATIONS_TO_INFER_DESER)) {
             return PropertyName.USE_DEFAULT;
         }
         return null;
@@ -1400,10 +1413,8 @@ public class JacksonAnnotationIntrospector
             if (info == null) {
                 return null;
             }
-            /* let's not try to force access override (would need to pass
-             * settings through if we did, since that's not doable on some
-             * platforms)
-             */
+            // let's not try to force access override (would need to pass
+            // settings through if we did, since that's not doable on some platforms)
             b = config.typeResolverBuilderInstance(ann, resAnn.value());
         } else { // if not, use standard one, if indicated by annotations
             if (info == null) {
@@ -1423,10 +1434,9 @@ public class JacksonAnnotationIntrospector
             idRes.init(baseType);
         }
         b = b.init(info.use(), idRes);
-        /* 13-Aug-2011, tatu: One complication; external id
-         *   only works for properties; so if declared for a Class, we will need
-         *   to map it to "PROPERTY" instead of "EXTERNAL_PROPERTY"
-         */
+        // 13-Aug-2011, tatu: One complication; external id only works for properties;
+        //    so if declared for a Class, we will need to map it to "PROPERTY"
+        //    instead of "EXTERNAL_PROPERTY"
         JsonTypeInfo.As inclusion = info.include();
         if (inclusion == JsonTypeInfo.As.EXTERNAL_PROPERTY && (ann instanceof AnnotatedClass)) {
             inclusion = JsonTypeInfo.As.PROPERTY;
