@@ -1,13 +1,14 @@
 package com.fasterxml.jackson.databind.ser.std;
 
 import java.io.IOException;
-import java.lang.reflect.ParameterizedType;
 import java.lang.reflect.Type;
 
+import com.fasterxml.jackson.annotation.JsonFormat;
+
 import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.core.type.WritableTypeId;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.introspect.AnnotatedMember;
-import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonArrayFormatVisitor;
 import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonFormatVisitorWrapper;
 import com.fasterxml.jackson.databind.jsonschema.SchemaAware;
 import com.fasterxml.jackson.databind.jsontype.TypeSerializer;
@@ -15,20 +16,34 @@ import com.fasterxml.jackson.databind.node.ObjectNode;
 import com.fasterxml.jackson.databind.ser.ContainerSerializer;
 import com.fasterxml.jackson.databind.ser.ContextualSerializer;
 import com.fasterxml.jackson.databind.ser.impl.PropertySerializerMap;
-import com.fasterxml.jackson.databind.type.TypeFactory;
 
 /**
  * Base class for serializers that will output contents as JSON
  * arrays; typically serializers used for {@link java.util.Collection}
  * and array types.
  */
+@SuppressWarnings("serial")
 public abstract class AsArraySerializerBase<T>
     extends ContainerSerializer<T>
     implements ContextualSerializer
 {
+    protected final JavaType _elementType;
+
+    /**
+     * Collection-valued property being serialized with this instance
+     */
+    protected final BeanProperty _property;
+
     protected final boolean _staticTyping;
 
-    protected final JavaType _elementType;
+    /**
+     * Setting for specific local override for "unwrap single element arrays":
+     * true for enable unwrapping, false for preventing it, `null` for using
+     * global configuration.
+     *
+     * @since 2.6
+     */
+    protected final Boolean _unwrapSingle;
 
     /**
      * Type serializer used for values, if any.
@@ -41,22 +56,42 @@ public abstract class AsArraySerializerBase<T>
     protected final JsonSerializer<Object> _elementSerializer;
 
     /**
-     * Collection-valued property being serialized with this instance
-     */
-    protected final BeanProperty _property;
-
-    /**
-     * If element type can not be statically determined, mapping from
+     * If element type cannot be statically determined, mapping from
      * runtime type to serializer is handled using this object
      */
     protected PropertySerializerMap _dynamicSerializers;
-    
+
     /*
     /**********************************************************
     /* Life-cycle
     /**********************************************************
      */
-    
+
+    /**
+     * Non-contextual, "blueprint" constructor typically called when the first
+     * instance is created, without knowledge of property it was used via.
+     *
+     * @since 2.6
+     */
+    protected AsArraySerializerBase(Class<?> cls, JavaType et, boolean staticTyping,
+            TypeSerializer vts, JsonSerializer<Object> elementSerializer)
+    {
+        super(cls, false);
+        _elementType = et;
+        // static if explicitly requested, or if element type is final
+        _staticTyping = staticTyping || (et != null && et.isFinal());
+        _valueTypeSerializer = vts;
+        _property = null;
+        _elementSerializer = elementSerializer;
+        _dynamicSerializers = PropertySerializerMap.emptyForProperties();
+        _unwrapSingle = null;
+    }
+
+    /**
+     * @deprecated Since 2.6 Use variants that either take 'src', or do NOT pass
+     *    BeanProperty
+     */
+    @Deprecated
     protected AsArraySerializerBase(Class<?> cls, JavaType et, boolean staticTyping,
             TypeSerializer vts, BeanProperty property, JsonSerializer<Object> elementSerializer)
     {
@@ -68,12 +103,14 @@ public abstract class AsArraySerializerBase<T>
         _valueTypeSerializer = vts;
         _property = property;
         _elementSerializer = elementSerializer;
-        _dynamicSerializers = PropertySerializerMap.emptyMap();
+        _dynamicSerializers = PropertySerializerMap.emptyForProperties();
+        _unwrapSingle = null;
     }
 
     @SuppressWarnings("unchecked")
     protected AsArraySerializerBase(AsArraySerializerBase<?> src,
-            BeanProperty property, TypeSerializer vts, JsonSerializer<?> elementSerializer)
+            BeanProperty property, TypeSerializer vts, JsonSerializer<?> elementSerializer,
+            Boolean unwrapSingle)
     {
         super(src);
         _elementType = src._elementType;
@@ -81,11 +118,36 @@ public abstract class AsArraySerializerBase<T>
         _valueTypeSerializer = vts;
         _property = property;
         _elementSerializer = (JsonSerializer<Object>) elementSerializer;
-        _dynamicSerializers = src._dynamicSerializers;
+        // [databind#2181]: may not be safe to reuse, start from empty
+        _dynamicSerializers = PropertySerializerMap.emptyForProperties();
+        _unwrapSingle = unwrapSingle;
+    }
+
+    /**
+     * @deprecated since 2.6: use the overloaded method that takes 'unwrapSingle'
+     */
+    @Deprecated
+    protected AsArraySerializerBase(AsArraySerializerBase<?> src,
+            BeanProperty property, TypeSerializer vts, JsonSerializer<?> elementSerializer)
+    {
+        this(src, property, vts, elementSerializer, src._unwrapSingle);
     }
     
+    /**
+     * @deprecated since 2.6: use the overloaded method that takes 'unwrapSingle'
+     */
+    @Deprecated
+    public final AsArraySerializerBase<T> withResolved(BeanProperty property,
+            TypeSerializer vts, JsonSerializer<?> elementSerializer) {
+        return withResolved(property, vts, elementSerializer, _unwrapSingle);
+    }
+
+    /**
+     * @since 2.6
+     */
     public abstract AsArraySerializerBase<T> withResolved(BeanProperty property,
-            TypeSerializer vts, JsonSerializer<?> elementSerializer);
+            TypeSerializer vts, JsonSerializer<?> elementSerializer,
+            Boolean unwrapSingle);
 
     /*
     /**********************************************************
@@ -100,7 +162,7 @@ public abstract class AsArraySerializerBase<T>
      * known statically.
      */
     @Override
-    public JsonSerializer<?> createContextual(SerializerProvider provider,
+    public JsonSerializer<?> createContextual(SerializerProvider serializers,
             BeanProperty property)
         throws JsonMappingException
     {
@@ -108,45 +170,47 @@ public abstract class AsArraySerializerBase<T>
         if (typeSer != null) {
             typeSer = typeSer.forProperty(property);
         }
-        /* 29-Sep-2012, tatu: Actually, we need to do much more contextual
-         *    checking here since we finally know for sure the property,
-         *    and it may have overrides
-         */
         JsonSerializer<?> ser = null;
+        Boolean unwrapSingle = null;
         // First: if we have a property, may have property-annotation overrides
+        
         if (property != null) {
+            final AnnotationIntrospector intr = serializers.getAnnotationIntrospector();
             AnnotatedMember m = property.getMember();
             if (m != null) {
-                Object serDef = provider.getAnnotationIntrospector().findContentSerializer(m);
+                Object serDef = intr.findContentSerializer(m);
                 if (serDef != null) {
-                    ser = provider.serializerInstance(m, serDef);
+                    ser = serializers.serializerInstance(m, serDef);
                 }
             }
         }
+        JsonFormat.Value format = findFormatOverrides(serializers, property, handledType());
+        if (format != null) {
+            unwrapSingle = format.getFeature(JsonFormat.Feature.WRITE_SINGLE_ELEM_ARRAYS_UNWRAPPED);
+        }
         if (ser == null) {
             ser = _elementSerializer;
         }
         // 18-Feb-2013, tatu: May have a content converter:
-        ser = findConvertingContentSerializer(provider, property, ser);
+        ser = findContextualConvertingSerializer(serializers, property, ser);
         if (ser == null) {
             // 30-Sep-2012, tatu: One more thing -- if explicit content type is annotated,
             //   we can consider it a static case as well.
             if (_elementType != null) {
-                // 20-Aug-2013, tatu: Need to avoid trying to access serializer for java.lang.Object tho
-                if ((_staticTyping && _elementType.getRawClass() != Object.class)
-                        || hasContentTypeAnnotation(provider, property)) {
-                    ser = provider.findValueSerializer(_elementType, property);
+                if (_staticTyping && !_elementType.isJavaLangObject()) {
+                    ser = serializers.findValueSerializer(_elementType, property);
                 }
             }
-        } else {
-            ser = provider.handleSecondaryContextualization(ser, property);
         }
-        if ((ser != _elementSerializer) || (property != _property) || _valueTypeSerializer != typeSer) {
-            return withResolved(property, typeSer, ser);
+        if ((ser != _elementSerializer)
+                || (property != _property)
+                || (_valueTypeSerializer != typeSer)
+                || (_unwrapSingle != unwrapSingle)) {
+            return withResolved(property, typeSer, ser, unwrapSingle);
         }
         return this;
     }
-    
+
     /*
     /**********************************************************
     /* Accessors
@@ -168,98 +232,70 @@ public abstract class AsArraySerializerBase<T>
     /* Serialization
     /**********************************************************
      */
-    
+
+    // NOTE: as of 2.5, sub-classes SHOULD override (in 2.4 and before, was final),
+    // at least if they can provide access to actual size of value and use `writeStartArray()`
+    // variant that passes size of array to output, which is helpful with some data formats
     @Override
-    public final void serialize(T value, JsonGenerator jgen, SerializerProvider provider)
-        throws IOException, JsonGenerationException
+    public void serialize(T value, JsonGenerator gen, SerializerProvider provider) throws IOException
     {
-        // [JACKSON-805]
         if (provider.isEnabled(SerializationFeature.WRITE_SINGLE_ELEM_ARRAYS_UNWRAPPED)
                 && hasSingleElement(value)) {
-            serializeContents(value, jgen, provider);
+            serializeContents(value, gen, provider);
             return;
         }
-        jgen.writeStartArray();
-        serializeContents(value, jgen, provider);
-        jgen.writeEndArray();
+        gen.writeStartArray(value);
+        serializeContents(value, gen, provider);
+        gen.writeEndArray();
     }
 
-    // Note: was 'final' modifier in 2.2 and before; no real need to be, removed
     @Override
-    public void serializeWithType(T value, JsonGenerator jgen, SerializerProvider provider,
-            TypeSerializer typeSer)
-        throws IOException, JsonGenerationException
+    public void serializeWithType(T value, JsonGenerator g, SerializerProvider provider,
+            TypeSerializer typeSer) throws IOException
     {
-        // note: let's NOT consider [JACKSON-805] here; gets too complicated, and probably just won't work
-        typeSer.writeTypePrefixForArray(value, jgen);
-        serializeContents(value, jgen, provider);
-        typeSer.writeTypeSuffixForArray(value, jgen);
+        WritableTypeId typeIdDef = typeSer.writeTypePrefix(g,
+                typeSer.typeId(value, JsonToken.START_ARRAY));
+        // [databind#631]: Assign current value, to be accessible by custom serializers
+        g.setCurrentValue(value);
+        serializeContents(value, g, provider);
+        typeSer.writeTypeSuffix(g, typeIdDef);
     }
 
-    protected abstract void serializeContents(T value, JsonGenerator jgen, SerializerProvider provider)
-        throws IOException, JsonGenerationException;
+    protected abstract void serializeContents(T value, JsonGenerator gen, SerializerProvider provider)
+        throws IOException;
 
     @SuppressWarnings("deprecation")
     @Override
     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
         throws JsonMappingException
     {
-        /* 15-Jan-2010, tatu: This should probably be rewritten, given that
-         *    more information about content type is actually being explicitly
-         *    passed. So there should be less need to try to re-process that
-         *    information.
-         */
         ObjectNode o = createSchemaNode("array", true);
-        JavaType contentType = null;
-        if (typeHint != null) {
-            JavaType javaType = provider.constructType(typeHint);
-            contentType = javaType.getContentType();
-            if (contentType == null) { // could still be parametrized (Iterators)
-                if (typeHint instanceof ParameterizedType) {
-                    Type[] typeArgs = ((ParameterizedType) typeHint).getActualTypeArguments();
-                    if (typeArgs.length == 1) {
-                        contentType = provider.constructType(typeArgs[0]);
-                    }
-                }
-            }
-        }
-        if (contentType == null && _elementType != null) {
-            contentType = _elementType;
-        }
-        if (contentType != null) {
+        if (_elementSerializer != null) {
             JsonNode schemaNode = null;
-            // 15-Oct-2010, tatu: We can't serialize plain Object.class; but what should it produce here? Untyped?
-            if (contentType.getRawClass() != Object.class) {
-                JsonSerializer<Object> ser = provider.findValueSerializer(contentType, _property);
-                if (ser instanceof SchemaAware) {
-                    schemaNode = ((SchemaAware) ser).getSchema(provider, null);
-                }
+            if (_elementSerializer instanceof SchemaAware) {
+                schemaNode = ((SchemaAware) _elementSerializer).getSchema(provider, null);
             }
             if (schemaNode == null) {
                 schemaNode = com.fasterxml.jackson.databind.jsonschema.JsonSchema.getDefaultSchemaNode();
             }
-            o.put("items", schemaNode);
+            o.set("items", schemaNode);
         }
         return o;
     }
-    
+
     @Override
     public void acceptJsonFormatVisitor(JsonFormatVisitorWrapper visitor, JavaType typeHint)
         throws JsonMappingException
     {
-        JsonArrayFormatVisitor arrayVisitor = (visitor == null) ? null : visitor.expectArrayFormat(typeHint);
-        if (arrayVisitor != null) {
-            TypeFactory tf = visitor.getProvider().getTypeFactory();
-            JavaType contentType = tf.moreSpecificType(_elementType, typeHint.getContentType());
-            if (contentType == null) {
-                throw new JsonMappingException("Could not resolve type");
+        JsonSerializer<?> valueSer = _elementSerializer;
+        if (valueSer == null) {
+            // 19-Oct-2016, tatu: Apparently we get null for untyped/raw `EnumSet`s... not 100%
+            //   sure what'd be the clean way but let's try this for now:
+            if (_elementType != null) {
+                valueSer = visitor.getProvider().findValueSerializer(_elementType, _property);
             }
-            JsonSerializer<?> valueSer = _elementSerializer;
-            if (valueSer == null) {
-                valueSer = visitor.getProvider().findValueSerializer(contentType, _property);
-            }
-            arrayVisitor.itemsFormat(valueSer, contentType);
         }
+        visitArrayFormat(visitor, typeHint, valueSer, _elementType);
     }
 
     protected final JsonSerializer<Object> _findAndAddDynamic(PropertySerializerMap map,
