@@ -4,12 +4,10 @@ import java.io.IOException;
 import java.lang.reflect.Type;
 import java.util.*;
 
-import com.fasterxml.jackson.annotation.JsonFormat;
-import com.fasterxml.jackson.annotation.ObjectIdGenerator;
-import com.fasterxml.jackson.annotation.ObjectIdGenerators;
+import com.fasterxml.jackson.annotation.*;
 import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.core.type.WritableTypeId;
 import com.fasterxml.jackson.databind.*;
-import com.fasterxml.jackson.databind.introspect.Annotated;
 import com.fasterxml.jackson.databind.introspect.AnnotatedMember;
 import com.fasterxml.jackson.databind.introspect.ObjectIdInfo;
 import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonFormatVisitable;
@@ -20,10 +18,12 @@ import com.fasterxml.jackson.databind.jsonschema.SchemaAware;
 import com.fasterxml.jackson.databind.jsontype.TypeSerializer;
 import com.fasterxml.jackson.databind.node.ObjectNode;
 import com.fasterxml.jackson.databind.ser.*;
+import com.fasterxml.jackson.databind.ser.impl.MapEntrySerializer;
 import com.fasterxml.jackson.databind.ser.impl.ObjectIdWriter;
 import com.fasterxml.jackson.databind.ser.impl.PropertyBasedObjectIdGenerator;
 import com.fasterxml.jackson.databind.ser.impl.WritableObjectId;
 import com.fasterxml.jackson.databind.util.ArrayBuilders;
+import com.fasterxml.jackson.databind.util.ClassUtil;
 import com.fasterxml.jackson.databind.util.Converter;
 import com.fasterxml.jackson.databind.util.NameTransformer;
 
@@ -33,13 +33,14 @@ import com.fasterxml.jackson.databind.util.NameTransformer;
  * Can be used for custom bean serializers as well, although that
  * is not the primary design goal.
  */
+@SuppressWarnings("serial")
 public abstract class BeanSerializerBase
     extends StdSerializer<Object>
     implements ContextualSerializer, ResolvableSerializer,
         JsonFormatVisitable, SchemaAware
 {
     protected final static PropertyName NAME_FOR_OBJECT_REF = new PropertyName("#object-ref");
-    
+
     final protected static BeanPropertyWriter[] NO_PROPS = new BeanPropertyWriter[0];
 
     /*
@@ -48,6 +49,11 @@ public abstract class BeanSerializerBase
     /**********************************************************
      */
 
+    /**
+     * @since 2.9
+     */
+    final protected JavaType _beanType;
+
     /**
      * Writers used for outputting actual property values
      */
@@ -75,7 +81,7 @@ public abstract class BeanSerializerBase
      * reference to that member.
      */
     final protected AnnotatedMember _typeId;
-    
+
     /**
      * If this POJO can be alternatively serialized using just an object id
      * to denote a reference to previously serialized object,
@@ -87,7 +93,7 @@ public abstract class BeanSerializerBase
      * Requested shape from bean class annotations.
      */
     final protected JsonFormat.Shape _serializationShape;
-    
+
     /*
     /**********************************************************
     /* Life-cycle: constructors
@@ -105,9 +111,12 @@ public abstract class BeanSerializerBase
             BeanPropertyWriter[] properties, BeanPropertyWriter[] filteredProperties)
     {
         super(type);
+        _beanType = type;
         _props = properties;
         _filteredProps = filteredProperties;
         if (builder == null) { // mostly for testing
+            // 20-Sep-2019, tatu: Actually not just that but also "dummy" serializer for
+            //     case of no bean properties, too
             _typeId = null;
             _anyGetterWriter = null;
             _propertyFilterId = null;
@@ -127,6 +136,7 @@ public abstract class BeanSerializerBase
             BeanPropertyWriter[] properties, BeanPropertyWriter[] filteredProperties)
     {
         super(src._handledType);
+        _beanType = src._beanType;
         _props = properties;
         _filteredProps = filteredProperties;
 
@@ -150,6 +160,7 @@ public abstract class BeanSerializerBase
             ObjectIdWriter objectIdWriter, Object filterId)
     {
         super(src._handledType);
+        _beanType = src._beanType;
         _props = src._props;
         _filteredProps = src._filteredProps;
         
@@ -160,12 +171,17 @@ public abstract class BeanSerializerBase
         _serializationShape = src._serializationShape;
     }
 
+    @Deprecated // since 2.8, remove soon
     protected BeanSerializerBase(BeanSerializerBase src, String[] toIgnore)
+    {
+        this(src, ArrayBuilders.arrayToSet(toIgnore));
+    }
+    
+    protected BeanSerializerBase(BeanSerializerBase src, Set<String> toIgnore)
     {
         super(src._handledType);
 
-        // Bit clumsy, but has to do:
-        HashSet<String> ignoredSet = ArrayBuilders.arrayToSet(toIgnore);
+        _beanType = src._beanType;
         final BeanPropertyWriter[] propsIn = src._props;
         final BeanPropertyWriter[] fpropsIn = src._filteredProps;
         final int len = propsIn.length;
@@ -176,7 +192,7 @@ public abstract class BeanSerializerBase
         for (int i = 0; i < len; ++i) {
             BeanPropertyWriter bpw = propsIn[i];
             // should be ignored?
-            if (ignoredSet.contains(bpw.getName())) {
+            if ((toIgnore != null) && toIgnore.contains(bpw.getName())) {
                 continue;
             }
             propsOut.add(bpw);
@@ -206,9 +222,20 @@ public abstract class BeanSerializerBase
      * Mutant factory used for creating a new instance with additional
      * set of properties to ignore (from properties this instance otherwise has)
      * 
-     * @since 2.0
+     * @since 2.8
      */
-    protected abstract BeanSerializerBase withIgnorals(String[] toIgnore);
+    protected abstract BeanSerializerBase withIgnorals(Set<String> toIgnore);
+    
+    /**
+     * Mutant factory used for creating a new instance with additional
+     * set of properties to ignore (from properties this instance otherwise has)
+     * 
+     * @deprecated since 2.8
+     */
+    @Deprecated
+    protected BeanSerializerBase withIgnorals(String[] toIgnore) {
+        return withIgnorals(ArrayBuilders.arrayToSet(toIgnore));
+    }
 
     /**
      * Mutant factory for creating a variant that output POJO as a
@@ -225,7 +252,8 @@ public abstract class BeanSerializerBase
      * 
      * @since 2.3
      */
-    protected abstract BeanSerializerBase withFilterId(Object filterId);
+    @Override
+    public abstract BeanSerializerBase withFilterId(Object filterId);
     
     /**
      * Copy-constructor that is useful for sub-classes that just want to
@@ -295,7 +323,7 @@ public abstract class BeanSerializerBase
             if (prop.hasSerializer()) {
                 continue;
             }
-            // [Issue#124]: allow use of converters
+            // [databind#124]: allow use of converters
             JsonSerializer<Object> ser = findConvertingSerializer(provider, prop);
             if (ser == null) {
                 // Was the serialization type hard-coded? If so, use it
@@ -304,7 +332,7 @@ public abstract class BeanSerializerBase
                 // It not, we can use declared return type if and only if declared type is final:
                 // if not, we don't really know the actual type until we get the instance.
                 if (type == null) {
-                    type = provider.constructType(prop.getGenericPropertyType());
+                    type = prop.getType();
                     if (!type.isFinal()) {
                         if (type.isContainerType() || type.containedTypeCount() > 0) {
                             prop.setNonTrivialBaseType(type);
@@ -329,18 +357,23 @@ public abstract class BeanSerializerBase
                     }
                 }
             }
-            prop.assignSerializer(ser);
-            // and maybe replace filtered property too? (see [JACKSON-364])
+            // and maybe replace filtered property too?
             if (i < filteredCount) {
                 BeanPropertyWriter w2 = _filteredProps[i];
                 if (w2 != null) {
                     w2.assignSerializer(ser);
+                    // 17-Mar-2017, tatu: Typically will lead to chained call to original property,
+                    //    which would lead to double set. Not a problem itself, except... unwrapping
+                    //    may require work to be done, which does lead to an actual issue.
+                    continue;
                 }
             }
+            prop.assignSerializer(ser);
         }
 
         // also, any-getter may need to be resolved
         if (_anyGetterWriter != null) {
+            // 23-Feb-2015, tatu: Misleading, as this actually triggers call to contextualization...
             _anyGetterWriter.resolve(provider);
         }
     }
@@ -358,45 +391,95 @@ public abstract class BeanSerializerBase
     {
         final AnnotationIntrospector intr = provider.getAnnotationIntrospector();
         if (intr != null) {
-            Object convDef = intr.findSerializationConverter(prop.getMember());
-            if (convDef != null) {
-                Converter<Object,Object> conv = provider.converterInstance(prop.getMember(), convDef);
-                JavaType delegateType = conv.getOutputType(provider.getTypeFactory());
-                JsonSerializer<?> ser = provider.findValueSerializer(delegateType, prop);
-                return new StdDelegatingSerializer(conv, delegateType, ser);
+            AnnotatedMember m = prop.getMember();
+            if (m != null) {
+                Object convDef = intr.findSerializationConverter(m);
+                if (convDef != null) {
+                    Converter<Object,Object> conv = provider.converterInstance(prop.getMember(), convDef);
+                    JavaType delegateType = conv.getOutputType(provider.getTypeFactory());
+                    // [databind#731]: Should skip if nominally java.lang.Object
+                    JsonSerializer<?> ser = delegateType.isJavaLangObject() ? null
+                            : provider.findValueSerializer(delegateType, prop);
+                    return new StdDelegatingSerializer(conv, delegateType, ser);
+                }
             }
         }
         return null;
     }
 
+    @SuppressWarnings("incomplete-switch")
     @Override
     public JsonSerializer<?> createContextual(SerializerProvider provider,
             BeanProperty property)
         throws JsonMappingException
     {
-        ObjectIdWriter oiw = _objectIdWriter;
-        String[] ignorals = null;
-        Object newFilterId = null;
         final AnnotationIntrospector intr = provider.getAnnotationIntrospector();
         final AnnotatedMember accessor = (property == null || intr == null)
                 ? null : property.getMember();
-        
-        // First: may have an override for Object Id:
+        final SerializationConfig config = provider.getConfig();
+
+        // Let's start with one big transmutation: Enums that are annotated
+        // to serialize as Objects may want to revert
+        JsonFormat.Value format = findFormatOverrides(provider, property, handledType());
+        JsonFormat.Shape shape = null;
+        if ((format != null) && format.hasShape()) {
+            shape = format.getShape();
+            // or, alternatively, asked to revert "back to" other representations...
+            if ((shape != JsonFormat.Shape.ANY) && (shape != _serializationShape)) {
+                if (ClassUtil.isEnumType(_handledType)) {
+                    switch (shape) {
+                    case STRING:
+                    case NUMBER:
+                    case NUMBER_INT:
+                        // 12-Oct-2014, tatu: May need to introspect full annotations... but
+                        //   for now, just do class ones
+                        BeanDescription desc = config.introspectClassAnnotations(_beanType);
+                        JsonSerializer<?> ser = EnumSerializer.construct(_beanType.getRawClass(),
+                                provider.getConfig(), desc, format);
+                        return provider.handlePrimaryContextualization(ser, property);
+                    }
+                // 16-Oct-2016, tatu: Ditto for `Map`, `Map.Entry` subtypes
+                } else if (shape == JsonFormat.Shape.NATURAL) {
+                    if (_beanType.isMapLikeType() && Map.class.isAssignableFrom(_handledType)) {
+                        ;
+                    } else if (Map.Entry.class.isAssignableFrom(_handledType)) {
+                        JavaType mapEntryType = _beanType.findSuperType(Map.Entry.class);
+
+                        JavaType kt = mapEntryType.containedTypeOrUnknown(0);
+                        JavaType vt = mapEntryType.containedTypeOrUnknown(1);
+
+                        // 16-Oct-2016, tatu: could have problems with type handling, as we do not
+                        //   see if "static" typing is needed, nor look for `TypeSerializer` yet...
+                        JsonSerializer<?> ser = new MapEntrySerializer(_beanType, kt, vt,
+                                false, null, property);
+                        return provider.handlePrimaryContextualization(ser, property);
+                    }
+                }
+            }
+        }
+
+        ObjectIdWriter oiw = _objectIdWriter;
+        Set<String> ignoredProps = null;
+        Object newFilterId = null;
+
+        // Then we may have an override for Object Id
         if (accessor != null) {
-            ignorals = intr.findPropertiesToIgnore(accessor);
+            JsonIgnoreProperties.Value ignorals = intr.findPropertyIgnorals(accessor);
+            if (ignorals != null) {
+                ignoredProps = ignorals.findIgnoredForSerialization();
+            }
             ObjectIdInfo objectIdInfo = intr.findObjectIdInfo(accessor);
             if (objectIdInfo == null) {
                 // no ObjectId override, but maybe ObjectIdRef?
                 if (oiw != null) {
-                    objectIdInfo = intr.findObjectReferenceInfo(accessor,
-                            new ObjectIdInfo(NAME_FOR_OBJECT_REF, null, null, null));
-                    oiw = _objectIdWriter.withAlwaysAsId(objectIdInfo.getAlwaysAsId());
+                    objectIdInfo = intr.findObjectReferenceInfo(accessor, null);
+                    if (objectIdInfo != null) {
+                        oiw = _objectIdWriter.withAlwaysAsId(objectIdInfo.getAlwaysAsId());
+                    }
                 }
             } else {
-                /* Ugh: mostly copied from BeanSerializerBase: but can't easily
-                 * change it to be able to move to SerializerProvider (where it
-                 * really belongs)
-                 */
+                // Ugh: mostly copied from BeanDeserializerBase: but can't easily change it
+                // to be able to move to SerializerProvider (where it really belongs)
                 
                 // 2.1: allow modifications by "id ref" annotations as well:
                 objectIdInfo = intr.findObjectReferenceInfo(accessor, objectIdInfo);
@@ -409,17 +492,17 @@ public abstract class BeanSerializerBase
                     String propName = objectIdInfo.getPropertyName().getSimpleName();
                     BeanPropertyWriter idProp = null;
 
-                    for (int i = 0, len = _props.length ;; ++i) {
+                    for (int i = 0, len = _props.length; ; ++i) {
                         if (i == len) {
-                            throw new IllegalArgumentException("Invalid Object Id definition for "+_handledType.getName()
-                                    +": can not find property with name '"+propName+"'");
+                            provider.reportBadDefinition(_beanType, String.format(
+                                    "Invalid Object Id definition for %s: cannot find property with name '%s'",
+                                    handledType().getName(), propName));
                         }
                         BeanPropertyWriter prop = _props[i];
                         if (propName.equals(prop.getName())) {
                             idProp = prop;
-                            /* Let's force it to be the first property to output
-                             * (although it may still get rearranged etc)
-                             */
+                            // Let's force it to be the first property to output
+                            // (although it may still get rearranged etc)
                             if (i > 0) { // note: must shuffle both regular properties and filtered
                                 System.arraycopy(_props, 0, _props, 1, i);
                                 _props[0] = idProp;
@@ -441,7 +524,6 @@ public abstract class BeanSerializerBase
                             objectIdInfo.getAlwaysAsId());
                 }
             }
-            
             // Or change Filter Id in use?
             Object filterId = intr.findFilterId(accessor);
             if (filterId != null) {
@@ -461,31 +543,33 @@ public abstract class BeanSerializerBase
             }
         }
         // And possibly add more properties to ignore
-        if (ignorals != null && ignorals.length != 0) {
-            contextual = contextual.withIgnorals(ignorals);
+        if ((ignoredProps != null) && !ignoredProps.isEmpty()) {
+            contextual = contextual.withIgnorals(ignoredProps);
         }
         if (newFilterId != null) {
             contextual = contextual.withFilterId(newFilterId);
         }
-        
-        // One more thing: are we asked to serialize POJO as array?
-        JsonFormat.Shape shape = null;
-        if (accessor != null) {
-            JsonFormat.Value format = intr.findFormat((Annotated) accessor);
-
-            if (format != null) {
-                shape = format.getShape();
-            }
-        }
         if (shape == null) {
             shape = _serializationShape;
         }
+        // last but not least; may need to transmute into as-array serialization
         if (shape == JsonFormat.Shape.ARRAY) {
-            contextual = contextual.asArraySerializer();
+            return contextual.asArraySerializer();
         }
         return contextual;
     }
-    
+
+    /*
+    /**********************************************************
+    /* Accessors
+    /**********************************************************
+     */
+
+    @Override
+    public Iterator<PropertyWriter> properties() {
+        return Arrays.<PropertyWriter>asList(_props).iterator();
+    }
+
     /*
     /**********************************************************
     /* Partial JsonSerializer implementation
@@ -499,115 +583,114 @@ public abstract class BeanSerializerBase
     
     // Main serialization method left unimplemented
     @Override
-    public abstract void serialize(Object bean, JsonGenerator jgen, SerializerProvider provider)
-        throws IOException, JsonGenerationException;
+    public abstract void serialize(Object bean, JsonGenerator gen, SerializerProvider provider)
+        throws IOException;
 
     // Type-info-augmented case implemented as it does not usually differ between impls
     @Override
-    public void serializeWithType(Object bean, JsonGenerator jgen,
+    public void serializeWithType(Object bean, JsonGenerator gen,
             SerializerProvider provider, TypeSerializer typeSer)
-        throws IOException, JsonGenerationException
+        throws IOException
     {
         if (_objectIdWriter != null) {
-            _serializeWithObjectId(bean, jgen, provider, typeSer);
+            gen.setCurrentValue(bean); // [databind#631]
+            _serializeWithObjectId(bean, gen, provider, typeSer);
             return;
         }
 
-        String typeStr = (_typeId == null) ? null :_customTypeId(bean);
-        if (typeStr == null) {
-            typeSer.writeTypePrefixForObject(bean, jgen);
-        } else {
-            typeSer.writeCustomTypePrefixForObject(bean, jgen, typeStr);
-        }
+        gen.setCurrentValue(bean); // [databind#631]
+        WritableTypeId typeIdDef = _typeIdDef(typeSer, bean, JsonToken.START_OBJECT);
+        typeSer.writeTypePrefix(gen, typeIdDef);
         if (_propertyFilterId != null) {
-            serializeFieldsFiltered(bean, jgen, provider);
+            serializeFieldsFiltered(bean, gen, provider);
         } else {
-            serializeFields(bean, jgen, provider);
-        }
-        if (typeStr == null) {
-            typeSer.writeTypeSuffixForObject(bean, jgen);
-        } else {
-            typeSer.writeCustomTypeSuffixForObject(bean, jgen, typeStr);
+            serializeFields(bean, gen, provider);
         }
+        typeSer.writeTypeSuffix(gen, typeIdDef);
     }
 
-    protected final void _serializeWithObjectId(Object bean,
-            JsonGenerator jgen, SerializerProvider provider,
-            boolean startEndObject)
-        throws IOException, JsonGenerationException
+    protected final void _serializeWithObjectId(Object bean, JsonGenerator gen, SerializerProvider provider,
+            boolean startEndObject) throws IOException
     {
         final ObjectIdWriter w = _objectIdWriter;
         WritableObjectId objectId = provider.findObjectId(bean, w.generator);
         // If possible, write as id already
-        if (objectId.writeAsId(jgen, provider, w)) {
+        if (objectId.writeAsId(gen, provider, w)) {
             return;
         }
         // If not, need to inject the id:
         Object id = objectId.generateId(bean);
         if (w.alwaysAsId) {
-            w.serializer.serialize(id, jgen, provider);
+            w.serializer.serialize(id, gen, provider);
             return;
         }
         if (startEndObject) {
-            jgen.writeStartObject();
+            gen.writeStartObject(bean);
         }
-        objectId.writeAsField(jgen, provider, w);
+        objectId.writeAsField(gen, provider, w);
         if (_propertyFilterId != null) {
-            serializeFieldsFiltered(bean, jgen, provider);
+            serializeFieldsFiltered(bean, gen, provider);
         } else {
-            serializeFields(bean, jgen, provider);
+            serializeFields(bean, gen, provider);
         }
         if (startEndObject) {
-            jgen.writeEndObject();
+            gen.writeEndObject();
         }
     }
     
-    protected final void _serializeWithObjectId(Object bean,
-            JsonGenerator jgen, SerializerProvider provider,
-            TypeSerializer typeSer)
-        throws IOException, JsonGenerationException
+    protected final void _serializeWithObjectId(Object bean, JsonGenerator gen, SerializerProvider provider,
+            TypeSerializer typeSer) throws IOException
     {
         final ObjectIdWriter w = _objectIdWriter;
         WritableObjectId objectId = provider.findObjectId(bean, w.generator);
         // If possible, write as id already
-        if (objectId.writeAsId(jgen, provider, w)) {
+        if (objectId.writeAsId(gen, provider, w)) {
             return;
         }
         // If not, need to inject the id:
         Object id = objectId.generateId(bean);
         if (w.alwaysAsId) {
-            w.serializer.serialize(id, jgen, provider);
+            w.serializer.serialize(id, gen, provider);
             return;
         }
-
-        _serializeObjectId(bean, jgen, provider, typeSer, objectId);
+        _serializeObjectId(bean, gen, provider, typeSer, objectId);
     }
 
-    protected  void _serializeObjectId(Object bean,
-            JsonGenerator jgen,SerializerProvider provider,
-            TypeSerializer typeSer, WritableObjectId objectId)
-        throws IOException, JsonProcessingException, JsonGenerationException {
+    protected  void _serializeObjectId(Object bean, JsonGenerator g,
+            SerializerProvider provider,
+            TypeSerializer typeSer, WritableObjectId objectId) throws IOException
+    {
         final ObjectIdWriter w = _objectIdWriter;
-        String typeStr = (_typeId == null) ? null :_customTypeId(bean);
-        if (typeStr == null) {
-            typeSer.writeTypePrefixForObject(bean, jgen);
-        } else {
-            typeSer.writeCustomTypePrefixForObject(bean, jgen, typeStr);
-        }
-        objectId.writeAsField(jgen, provider, w);
+        WritableTypeId typeIdDef = _typeIdDef(typeSer, bean, JsonToken.START_OBJECT);
+
+        typeSer.writeTypePrefix(g, typeIdDef);
+        objectId.writeAsField(g, provider, w);
         if (_propertyFilterId != null) {
-            serializeFieldsFiltered(bean, jgen, provider);
+            serializeFieldsFiltered(bean, g, provider);
         } else {
-            serializeFields(bean, jgen, provider);
-        }
-        if (typeStr == null) {
-            typeSer.writeTypeSuffixForObject(bean, jgen);
-        } else {
-            typeSer.writeCustomTypeSuffixForObject(bean, jgen, typeStr);
+            serializeFields(bean, g, provider);
         }
+        typeSer.writeTypeSuffix(g, typeIdDef);
     }
-    
-    private final String _customTypeId(Object bean)
+
+    /**
+     * @since 2.9
+     */
+    protected final WritableTypeId _typeIdDef(TypeSerializer typeSer,
+            Object bean, JsonToken valueShape) {
+        if (_typeId == null) {
+            return typeSer.typeId(bean, valueShape);
+        }
+        Object typeId = _typeId.getValue(bean);
+        if (typeId == null) {
+            // 28-Jun-2017, tatu: Is this really needed? Unchanged from 2.8 but...
+            typeId = "";
+        }
+        return typeSer.typeId(bean, valueShape, typeId);
+    }
+
+    @Deprecated // since 2.9
+    protected final String _customTypeId(Object bean)
     {
         final Object typeId = _typeId.getValue(bean);
         if (typeId == null) {
@@ -615,15 +698,15 @@ public abstract class BeanSerializerBase
         }
         return (typeId instanceof String) ? (String) typeId : typeId.toString();
     }
-    
+
     /*
     /**********************************************************
     /* Field serialization methods
     /**********************************************************
      */
 
-    protected void serializeFields(Object bean, JsonGenerator jgen, SerializerProvider provider)
-        throws IOException, JsonGenerationException
+    protected void serializeFields(Object bean, JsonGenerator gen, SerializerProvider provider)
+        throws IOException
     {
         final BeanPropertyWriter[] props;
         if (_filteredProps != null && provider.getActiveView() != null) {
@@ -636,22 +719,24 @@ public abstract class BeanSerializerBase
             for (final int len = props.length; i < len; ++i) {
                 BeanPropertyWriter prop = props[i];
                 if (prop != null) { // can have nulls in filtered list
-                    prop.serializeAsField(bean, jgen, provider);
+                    prop.serializeAsField(bean, gen, provider);
                 }
             }
             if (_anyGetterWriter != null) {
-                _anyGetterWriter.getAndSerialize(bean, jgen, provider);
+                _anyGetterWriter.getAndSerialize(bean, gen, provider);
             }
         } catch (Exception e) {
             String name = (i == props.length) ? "[anySetter]" : props[i].getName();
             wrapAndThrow(provider, e, bean, name);
         } catch (StackOverflowError e) {
-            /* 04-Sep-2009, tatu: Dealing with this is tricky, since we do not
-             *   have many stack frames to spare... just one or two; can't
-             *   make many calls.
-             */
-            JsonMappingException mapE = new JsonMappingException("Infinite recursion (StackOverflowError)", e);
-            String name = (i == props.length) ? "[anySetter]" : props[i].getName();
+            // 04-Sep-2009, tatu: Dealing with this is tricky, since we don't have many
+            //   stack frames to spare... just one or two; can't make many calls.
+
+            // 10-Dec-2015, tatu: and due to above, avoid "from" method, call ctor directly:
+            //JsonMappingException mapE = JsonMappingException.from(gen, "Infinite recursion (StackOverflowError)", e);
+            JsonMappingException mapE = new JsonMappingException(gen, "Infinite recursion (StackOverflowError)", e);
+
+             String name = (i == props.length) ? "[anySetter]" : props[i].getName();
             mapE.prependPath(new JsonMappingException.Reference(bean, name));
             throw mapE;
         }
@@ -662,7 +747,7 @@ public abstract class BeanSerializerBase
      * {@link PropertyFilter} that needs to be called to determine
      * which properties are to be serialized (and possibly how)
      */
-    protected void serializeFieldsFiltered(Object bean, JsonGenerator jgen,
+    protected void serializeFieldsFiltered(Object bean, JsonGenerator gen,
             SerializerProvider provider)
         throws IOException, JsonGenerationException
     {
@@ -678,7 +763,7 @@ public abstract class BeanSerializerBase
         final PropertyFilter filter = findPropertyFilter(provider, _propertyFilterId, bean);
         // better also allow missing filter actually..
         if (filter == null) {
-            serializeFields(bean, jgen, provider);
+            serializeFields(bean, gen, provider);
             return;
         }
         int i = 0;
@@ -686,17 +771,19 @@ public abstract class BeanSerializerBase
             for (final int len = props.length; i < len; ++i) {
                 BeanPropertyWriter prop = props[i];
                 if (prop != null) { // can have nulls in filtered list
-                    filter.serializeAsField(bean, jgen, provider, prop);
+                    filter.serializeAsField(bean, gen, provider, prop);
                 }
             }
             if (_anyGetterWriter != null) {
-                _anyGetterWriter.getAndFilter(bean, jgen, provider, filter);
+                _anyGetterWriter.getAndFilter(bean, gen, provider, filter);
             }
         } catch (Exception e) {
             String name = (i == props.length) ? "[anySetter]" : props[i].getName();
             wrapAndThrow(provider, e, bean, name);
         } catch (StackOverflowError e) {
-            JsonMappingException mapE = new JsonMappingException("Infinite recursion (StackOverflowError)", e);
+            // Minimize call depth since we are close to fail:
+            //JsonMappingException mapE = JsonMappingException.from(gen, "Infinite recursion (StackOverflowError)", e);
+            JsonMappingException mapE = new JsonMappingException(gen, "Infinite recursion (StackOverflowError)", e);
             String name = (i == props.length) ? "[anySetter]" : props[i].getName();
             mapE.prependPath(new JsonMappingException.Reference(bean, name));
             throw mapE;
@@ -738,7 +825,7 @@ public abstract class BeanSerializerBase
             }
 
         }
-        o.put("properties", propertiesNode);
+        o.set("properties", propertiesNode);
         return o;
     }
     
@@ -754,15 +841,28 @@ public abstract class BeanSerializerBase
         if (objectVisitor == null) {
             return;
         }
+        final SerializerProvider provider = visitor.getProvider();
         if (_propertyFilterId != null) {
             PropertyFilter filter = findPropertyFilter(visitor.getProvider(),
                     _propertyFilterId, null);
-            for (int i = 0; i < _props.length; i++) {
-                filter.depositSchemaProperty(_props[i], objectVisitor, visitor.getProvider());
+            for (int i = 0, end = _props.length; i < end; ++i) {
+                filter.depositSchemaProperty(_props[i], objectVisitor, provider);
             }
         } else {
-            for (int i = 0; i < _props.length; i++) {
-                _props[i].depositSchemaProperty(objectVisitor);
+            Class<?> view = ((_filteredProps == null) || (provider == null))
+                    ? null : provider.getActiveView();
+            final BeanPropertyWriter[] props;
+            if (view != null) {
+                props = _filteredProps;
+            } else {
+                props = _props;
+            }
+
+            for (int i = 0, end = props.length; i < end; ++i) {
+                BeanPropertyWriter prop = props[i];
+                if (prop != null) { // may be filtered out unconditionally
+                    prop.depositSchemaProperty(objectVisitor, provider);
+                }
             }
         }
     }
