@@ -16,6 +16,13 @@ import com.fasterxml.jackson.core.io.NumberInput;
  * serializers and deserializers. For serialization defaults to using
  * an ISO-8601 compliant format (format String "yyyy-MM-dd'T'HH:mm:ss.SSSZ")
  * and for deserialization, both ISO-8601 and RFC-1123.
+ *<br>
+ * Note that `Z` in format String refers to RFC-822 timezone notation which produces
+ * values like "-0800" -- that is, full minute/hour combo without colon, and not using `Z`
+ * as alias for "+0000".
+ *<p>
+ * Note also that to enable use of colon in timezone is possible by using method
+ * {@link #withColonInTimeZone} for creating new differently configured format instance.
  */
 @SuppressWarnings("serial")
 public class StdDateFormat
@@ -145,6 +152,8 @@ public class StdDateFormat
 
     /** 
      * Whether the TZ offset must be formatted with a colon between hours and minutes ({@code HH:mm} format)
+     *<p>
+     * Defaults to {@code false} for backwards compatibility reasons
      *
      * @since 2.9.1
      */
@@ -434,8 +443,23 @@ public class StdDateFormat
     {
         Calendar cal = _getCalendar(tz);
         cal.setTime(date);
+        // [databind#2167]: handle range beyond [1, 9999]
+        final int year = cal.get(Calendar.YEAR);
 
-        pad4(buffer, cal.get(Calendar.YEAR));
+        // Assuming GregorianCalendar, special handling needed for BCE (aka BC)
+        if (cal.get(Calendar.ERA) == GregorianCalendar.BC) {
+            _formatBCEYear(buffer, year);
+        } else {
+            if (year > 9999) {
+                // 22-Nov-2018, tatu: Handling beyond 4-digits is not well specified wrt ISO-8601, but
+                //   it seems that plus prefix IS mandated. Padding is an open question, but since agreeement
+                //   for max length would be needed, we ewould need to limit to arbitrary length
+                //   like five digits (erroring out if beyond or padding to that as minimum).
+                //   Instead, let's just print number out as is and let decoder try to make sense of it.
+                buffer.append('+');
+            }
+            pad4(buffer, year);
+        }
         buffer.append('-');
         pad2(buffer, cal.get(Calendar.MONTH) + 1);
         buffer.append('-');
@@ -472,6 +496,21 @@ public class StdDateFormat
         }
     }
 
+    protected void _formatBCEYear(StringBuffer buffer, int bceYearNoSign) {
+        // Ok. First of all, BCE 1 output (given as value `1` in era BCE) needs to become
+        // "+0000", but rest (from `2` up, in that era) need minus sign.
+        if (bceYearNoSign == 1) {
+            buffer.append("+0000");
+            return;
+        }
+        final int isoYear = bceYearNoSign - 1;
+        buffer.append('-');
+        // as with CE, 4 digit variant needs padding; beyond that not (although that part is
+        // open to debate, needs agreement with receiver)
+        // But `pad4()` deals with "big" numbers now so:
+        pad4(buffer, isoYear);
+    }
+
     private static void pad2(StringBuffer buffer, int value) {
         int tens = value / 10;
         if (tens == 0) {
@@ -499,7 +538,11 @@ public class StdDateFormat
         if (h == 0) {
             buffer.append('0').append('0');
         } else {
-            pad2(buffer, h);
+            if (h > 99) { // [databind#2167]: handle above 9999 correctly
+                buffer.append(h);
+            } else {
+                pad2(buffer, h);
+            }
             value -= (100 * h);
         }
         pad2(buffer, value);
