@@ -6,11 +6,14 @@ import java.util.*;
 import static org.junit.Assert.*;
 
 import com.fasterxml.jackson.annotation.JsonCreator;
-import com.fasterxml.jackson.annotation.JsonValue;
+
+import com.fasterxml.jackson.core.*;
 
 import com.fasterxml.jackson.databind.ObjectMapper;
-
-import com.fasterxml.jackson.test.BaseTest;
+import com.fasterxml.jackson.databind.deser.std.StdScalarDeserializer;
+import com.fasterxml.jackson.databind.json.JsonMapper;
+import com.fasterxml.jackson.databind.ser.std.StdScalarSerializer;
+import com.fasterxml.jackson.databind.type.TypeFactory;
 
 public abstract class BaseMapTest
     extends BaseTest
@@ -23,6 +26,13 @@ public abstract class BaseMapTest
     /**********************************************************
      */
 
+    public static class BogusSchema implements FormatSchema {
+        @Override
+        public String getSchemaType() {
+            return "TestFormat";
+        }
+    }
+
     /**
      * Simple wrapper around boolean types, usually to test value
      * conversions or wrapping
@@ -30,10 +40,8 @@ public abstract class BaseMapTest
     protected static class BooleanWrapper {
         public Boolean b;
 
-        @JsonCreator
+        public BooleanWrapper() { }
         public BooleanWrapper(Boolean value) { b = value; }
-
-        @JsonValue public Boolean value() { return b; }
     }
 
     protected static class IntWrapper {
@@ -42,6 +50,20 @@ public abstract class BaseMapTest
         public IntWrapper() { }
         public IntWrapper(int value) { i = value; }
     }
+
+    protected static class LongWrapper {
+        public long l;
+
+        public LongWrapper() { }
+        public LongWrapper(long value) { l = value; }
+    }
+
+    protected static class DoubleWrapper {
+        public double d;
+
+        public DoubleWrapper() { }
+        public DoubleWrapper(double value) { d = value; }
+    }
     
     /**
      * Simple wrapper around String type, usually to test value
@@ -57,7 +79,7 @@ public abstract class BaseMapTest
     }
 
     protected static class ObjectWrapper {
-        private final Object object;
+        final Object object;
         protected ObjectWrapper(final Object object) {
             this.object = object;
         }
@@ -72,7 +94,7 @@ public abstract class BaseMapTest
     {
         public List<T> list;
 
-        public ListWrapper(T... values) {
+        public ListWrapper(@SuppressWarnings("unchecked") T... values) {
             list = new ArrayList<T>();
             for (T value : values) {
                 list.add(value);
@@ -84,9 +106,14 @@ public abstract class BaseMapTest
     {
         public Map<K,V> map;
 
+        public MapWrapper() { }
         public MapWrapper(Map<K,V> m) {
             map = m;
         }
+        public MapWrapper(K key, V value) {
+            map = new LinkedHashMap<>();
+            map.put(key, value);
+        }
     }
     
     protected static class ArrayWrapper<T>
@@ -109,30 +136,116 @@ public abstract class BaseMapTest
         public abstract void foobar();
     }
 
+    public enum ABC { A, B, C; }
+
+    // since 2.8
+    public static class Point {
+        public int x, y;
+
+        protected Point() { } // for deser
+        public Point(int x0, int y0) {
+            x = x0;
+            y = y0;
+        }
+    
+        @Override
+        public boolean equals(Object o) {
+            if (!(o instanceof Point)) {
+                return false;
+            }
+            Point other = (Point) o;
+            return (other.x == x) && (other.y == y);
+        }
+
+        @Override
+        public String toString() {
+            return String.format("[x=%d, y=%d]", x, y);
+        }
+    }
+
+    /*
+    /**********************************************************
+    /* Shared serializers
+    /**********************************************************
+     */
+
+    @SuppressWarnings("serial")
+    public static class UpperCasingSerializer extends StdScalarSerializer<String>
+    {
+        public UpperCasingSerializer() { super(String.class); }
+
+        @Override
+        public void serialize(String value, JsonGenerator gen,
+                SerializerProvider provider) throws IOException {
+            gen.writeString(value.toUpperCase());
+        }
+    }
+
+    @SuppressWarnings("serial")
+    public static class LowerCasingDeserializer extends StdScalarDeserializer<String>
+    {
+        public LowerCasingDeserializer() { super(String.class); }
+
+        @Override
+        public String deserialize(JsonParser p, DeserializationContext ctxt)
+                throws IOException, JsonProcessingException {
+            return p.getText().toLowerCase();
+        }
+    }
+
+    /*
+    /**********************************************************
+    /* Construction
+    /**********************************************************
+     */
+    
     protected BaseMapTest() { super(); }
 
     /*
     /**********************************************************
-    /* Additional assert methods
+    /* Factory methods
     /**********************************************************
      */
 
-    private final static ObjectMapper SHARED_MAPPER = new ObjectMapper();
+    private static ObjectMapper SHARED_MAPPER;
 
-    protected ObjectMapper objectMapper() {
+    protected ObjectMapper sharedMapper() {
+        if (SHARED_MAPPER == null) {
+            SHARED_MAPPER = newJsonMapper();
+        }
         return SHARED_MAPPER;
     }
-    
+
+    protected ObjectMapper objectMapper() {
+        return sharedMapper();
+    }
+
     protected ObjectWriter objectWriter() {
-        return SHARED_MAPPER.writer();
+        return sharedMapper().writer();
     }
 
     protected ObjectReader objectReader() {
-        return SHARED_MAPPER.reader();
+        return sharedMapper().reader();
     }
     
     protected ObjectReader objectReader(Class<?> cls) {
-        return SHARED_MAPPER.reader(cls);
+        return sharedMapper().readerFor(cls);
+    }
+
+    // @since 2.10
+    protected static ObjectMapper newJsonMapper() {
+        return new ObjectMapper();
+    }
+
+    // @since 2.10
+    protected static JsonMapper.Builder jsonMapperBuilder() {
+        return JsonMapper.builder();
+    }
+
+    // @since 2.7
+    protected TypeFactory newTypeFactory() {
+        // this is a work-around; no null modifier added
+        return TypeFactory.defaultInstance().withModifier(null);
     }
 
     /*
@@ -146,6 +259,11 @@ public abstract class BaseMapTest
         assertArrayEquals(exp, act);
     }
 
+    protected void assertEquals(byte[] exp, byte[] act)
+    {
+        assertArrayEquals(exp, act);
+    }
+
     /**
      * Helper method for verifying 3 basic cookie cutter cases;
      * identity comparison (true), and against null (false),
@@ -183,13 +301,13 @@ public abstract class BaseMapTest
     protected String serializeAsString(Object value)
         throws IOException
     {
-        return serializeAsString(SHARED_MAPPER, value);
+        return serializeAsString(sharedMapper(), value);
     }
 
     protected String asJSONObjectValueString(Object... args)
         throws IOException
     {
-        return asJSONObjectValueString(SHARED_MAPPER, args);
+        return asJSONObjectValueString(sharedMapper(), args);
     }
 
     protected String asJSONObjectValueString(ObjectMapper m, Object... args)
@@ -211,11 +329,10 @@ public abstract class BaseMapTest
     protected <T> T readAndMapFromString(String input, Class<T> cls)
         throws IOException
     {
-        return readAndMapFromString(SHARED_MAPPER, input, cls);
+        return readAndMapFromString(sharedMapper(), input, cls);
     }
-    
-    protected <T> T readAndMapFromString(ObjectMapper m, String input, Class<T> cls)
-        throws IOException
+
+    protected <T> T readAndMapFromString(ObjectMapper m, String input, Class<T> cls) throws IOException
     {
         return (T) m.readValue("\""+input+"\"", cls);
     }
@@ -225,7 +342,7 @@ public abstract class BaseMapTest
     /* Helper methods, other
     /**********************************************************
      */
-    
+
     protected TimeZone getUTCTimeZone() {
         return TimeZone.getTimeZone("GMT");
     }
@@ -238,7 +355,11 @@ public abstract class BaseMapTest
         }
     }
 
-    protected String aposToQuotes(String json) {
+    protected static String aposToQuotes(String json) {
         return json.replace("'", "\"");
     }
+
+    protected static String quotesToApos(String json) {
+        return json.replace("\"", "'");
+    }
 }
