@@ -7,12 +7,15 @@ import java.util.concurrent.TimeUnit;
 import com.fasterxml.jackson.annotation.*;
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.type.TypeReference;
+
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
 import com.fasterxml.jackson.databind.deser.std.FromStringDeserializer;
 import com.fasterxml.jackson.databind.deser.std.StdDeserializer;
 import com.fasterxml.jackson.databind.exc.InvalidFormatException;
 import com.fasterxml.jackson.databind.exc.MismatchedInputException;
+import com.fasterxml.jackson.databind.exc.ValueInstantiationException;
+import com.fasterxml.jackson.databind.json.JsonMapper;
 import com.fasterxml.jackson.databind.module.SimpleModule;
 
 @SuppressWarnings("serial")
@@ -183,6 +186,35 @@ public class EnumDeserializationTest
         }
     }
 
+    // for [databind#2164]
+    public enum TestEnum2164 {
+        A, B;
+
+        @JsonCreator
+        public static TestEnum2164 fromString(String input) {
+            throw new IllegalArgumentException("2164");
+        }
+    }
+
+    // for [databind#2309]
+    static enum Enum2309 {
+        NON_NULL("NON_NULL"),
+        NULL(null),
+        OTHER("OTHER")
+        ;
+
+        private String value;
+
+        private Enum2309(String value) {
+            this.value = value;
+        }
+
+        @Override
+        public String toString() {
+            return value;
+        }
+    }        
+
     /*
     /**********************************************************
     /* Test methods
@@ -215,7 +247,7 @@ public class EnumDeserializationTest
             /*Object result =*/ MAPPER.readValue("\"NO-SUCH-VALUE\"", TestEnum.class);
             fail("Expected an exception for bogus enum value...");
         } catch (MismatchedInputException jex) {
-            verifyException(jex, "value not one of declared");
+            verifyException(jex, "not one of the values accepted for Enum class");
         }
         jp.close();
     }
@@ -284,17 +316,17 @@ public class EnumDeserializationTest
         } catch (MismatchedInputException e) {
             verifyException(e, "Cannot deserialize");
             // 26-Jan-2017, tatu: as per [databind#1505], should fail bit differently
-            verifyException(e, "value not one of declared Enum");
+            verifyException(e, "not one of the values accepted for Enum class");
         }
     }
 
     public void testEnumsWithIndex() throws Exception
     {
-        ObjectMapper m = new ObjectMapper();
-        m.enable(SerializationFeature.WRITE_ENUMS_USING_INDEX);
-        String json = m.writeValueAsString(TestEnum.RULES);
+        String json = MAPPER.writer()
+                .with(SerializationFeature.WRITE_ENUMS_USING_INDEX)
+                .writeValueAsString(TestEnum.RULES);
         assertEquals(String.valueOf(TestEnum.RULES.ordinal()), json);
-        TestEnum result = m.readValue(json, TestEnum.class);
+        TestEnum result = MAPPER.readValue(json, TestEnum.class);
         assertSame(TestEnum.RULES, result);
     }
 
@@ -391,14 +423,14 @@ public class EnumDeserializationTest
 
     // [databind#381]
     public void testUnwrappedEnum() throws Exception {
-        final ObjectMapper mapper = newObjectMapper();
-        mapper.enable(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS);
-        
-        assertEquals(TestEnum.JACKSON, mapper.readValue("[" + quote("JACKSON") + "]", TestEnum.class));
+        assertEquals(TestEnum.JACKSON,
+                MAPPER.readerFor(TestEnum.class)
+                    .with(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS)
+                    .readValue("[" + quote("JACKSON") + "]"));
     }
     
     public void testUnwrappedEnumException() throws Exception {
-        final ObjectMapper mapper = newObjectMapper();
+        final ObjectMapper mapper = newJsonMapper();
         mapper.disable(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS);
         try {
             Object v = mapper.readValue("[" + quote("JACKSON") + "]",
@@ -422,10 +454,12 @@ public class EnumDeserializationTest
         assertSame(TestEnum.values()[1], en);
 
         // [databind#1690]: unless prevented
-        final ObjectMapper mapper = newObjectMapper();
-        mapper.disable(MapperFeature.ALLOW_COERCION_OF_SCALARS);
         try {
-            en = mapper.readValue(quote("1"), TestEnum.class);
+            en = JsonMapper.builder()
+                    .configure(MapperFeature.ALLOW_COERCION_OF_SCALARS, false)
+                    .build()
+                    .readerFor(TestEnum.class)
+                    .readValue(quote("1"));
             fail("Should not pass");
         } catch (MismatchedInputException e) {
             verifyException(e, "Cannot deserialize value of type");
@@ -517,7 +551,7 @@ public class EnumDeserializationTest
     }
 
     public void testExceptionFromCustomEnumKeyDeserializer() throws Exception {
-        ObjectMapper mapper = newObjectMapper()
+        ObjectMapper mapper = newJsonMapper()
                 .registerModule(new EnumModule());
         try {
             mapper.readValue("{\"TWO\": \"dumpling\"}",
@@ -527,4 +561,39 @@ public class EnumDeserializationTest
             assertTrue(e.getMessage().contains("Undefined AnEnum"));
         }
     }
+
+    // [databind#2164]
+    public void testWrapExceptions() throws Exception
+    {
+        // By default, wrap:
+        try {
+            MAPPER.readerFor(TestEnum2164.class)
+                .readValue(quote("B"));
+            fail("Should not pass");
+        } catch (ValueInstantiationException e) {
+            verifyException(e, "2164");
+        }
+
+        // But can disable:
+        try {
+            MAPPER.readerFor(TestEnum2164.class)
+                .without(DeserializationFeature.WRAP_EXCEPTIONS)
+                .readValue(quote("B"));
+            fail("Should not pass");
+        } catch (JsonMappingException e) {
+            fail("Wrong exception, should not wrap, got: "+e);
+        } catch (IllegalArgumentException e) {
+            verifyException(e, "2164");
+        }
+    }
+
+    // [databind#2309]
+    public void testEnumToStringNull2309() throws Exception
+    {
+        Enum2309 value = MAPPER.readerFor(Enum2309.class)
+                .with(DeserializationFeature.READ_ENUMS_USING_TO_STRING)
+                .readValue(quote("NON_NULL"));
+        assertEquals(Enum2309.NON_NULL, value);
+    }
+
 }
