@@ -33,7 +33,9 @@ public class TestInferredMutators extends BaseMapTest
         ObjectMapper mapper = new ObjectMapper();
         // default value is 'enabled', for backwards compatibility
         assertTrue(mapper.isEnabled(MapperFeature.ALLOW_FINAL_FIELDS_AS_MUTATORS));
-        mapper.disable(MapperFeature.ALLOW_FINAL_FIELDS_AS_MUTATORS);
+        mapper = jsonMapperBuilder()
+                .disable(MapperFeature.ALLOW_FINAL_FIELDS_AS_MUTATORS)
+                .build();
         try {
             /*p =*/ mapper.readValue("{\"x\":2}", FixedPoint.class);
             fail("Should not try to use final field");
@@ -53,8 +55,9 @@ public class TestInferredMutators extends BaseMapTest
         assertEquals(2, p.x);
 
         // but without it, should fail:
-        mapper = new ObjectMapper();
-        mapper.disable(MapperFeature.INFER_PROPERTY_MUTATORS);
+        mapper = jsonMapperBuilder()
+                .disable(MapperFeature.INFER_PROPERTY_MUTATORS)
+                .build();
         try {
             p = mapper.readValue(JSON,  Point.class);
             fail("Should not succeeed");
