@@ -9,12 +9,13 @@ import com.fasterxml.jackson.annotation.*;
 import com.fasterxml.jackson.annotation.JsonSubTypes.Type;
 import com.fasterxml.jackson.annotation.JsonTypeInfo.As;
 import com.fasterxml.jackson.annotation.JsonTypeInfo.Id;
+
 import com.fasterxml.jackson.core.type.TypeReference;
+
 import com.fasterxml.jackson.databind.BaseMapTest;
 import com.fasterxml.jackson.databind.JavaType;
 import com.fasterxml.jackson.databind.ObjectMapper;
 import com.fasterxml.jackson.databind.annotation.JsonSerialize;
-import com.fasterxml.jackson.databind.type.TypeFactory;
 
 public class TestTypedContainerSerialization
 	extends BaseMapTest
@@ -123,13 +124,13 @@ public class TestTypedContainerSerialization
 
     public void testIssue329() throws Exception
     {
-            ArrayList<Animal> animals = new ArrayList<Animal>();
-            animals.add(new Dog("Spot"));
-            JavaType rootType = TypeFactory.defaultInstance().constructParametricType(Iterator.class, Animal.class);
-            String json = mapper.writerWithType(rootType).writeValueAsString(animals.iterator());
-            if (json.indexOf("\"object-type\":\"doggy\"") < 0) {
-                fail("No polymorphic type retained, should be; JSON = '"+json+"'");
-            }
+        ArrayList<Animal> animals = new ArrayList<Animal>();
+        animals.add(new Dog("Spot"));
+        JavaType rootType = mapper.getTypeFactory().constructParametricType(Iterator.class, Animal.class);
+        String json = mapper.writerFor(rootType).writeValueAsString(animals.iterator());
+        if (json.indexOf("\"object-type\":\"doggy\"") < 0) {
+            fail("No polymorphic type retained, should be; JSON = '"+json+"'");
+        }
     }
 
     public void testIssue508() throws Exception
@@ -138,8 +139,8 @@ public class TestTypedContainerSerialization
             List<Issue508A> l2 = new ArrayList<Issue508A>();
             l2.add(new Issue508A());
             l.add(l2);
-            TypeReference<?> typeRef = new TypeReference<List<List<Issue508A>>>() {};
-            String json = mapper.writerWithType(typeRef).writeValueAsString(l);
+            TypeReference<List<List<Issue508A>>> typeRef = new TypeReference<List<List<Issue508A>>>() {};
+            String json = mapper.writerFor(typeRef).writeValueAsString(l);
 
             List<?> output = mapper.readValue(json, typeRef);
             assertEquals(1, output.size());
