@@ -3,13 +3,9 @@ package com.fasterxml.jackson.databind.ser;
 import java.io.*;
 import java.util.*;
 
-import com.fasterxml.jackson.annotation.JsonFormat;
-import com.fasterxml.jackson.annotation.JsonFormat.Shape;
-import com.fasterxml.jackson.annotation.JsonProperty;
-import com.fasterxml.jackson.annotation.JsonValue;
+import com.fasterxml.jackson.annotation.*;
 
 import com.fasterxml.jackson.core.*;
-
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JsonSerialize;
 import com.fasterxml.jackson.databind.jsontype.TypeSerializer;
@@ -24,12 +20,6 @@ import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
 public class TestEnumSerialization
     extends BaseMapTest
 {
-    /*
-    /**********************************************************
-    /* Helper enums
-    /**********************************************************
-     */
-
     /**
      * Test enumeration for verifying Enum serialization functionality.
      */
@@ -57,17 +47,20 @@ public class TestEnumSerialization
         private EnumWithJsonValue(String n) {
             name = n;
         }
-        @JsonValue
+
         @Override
         public String toString() { return name; }
+
+        @JsonValue
+        public String external() { return "value:"+name; }
     }
-    
+
     protected static interface ToStringMixin {
         @Override
         @JsonValue public String toString();
     }
 
-    protected enum SerializableEnum implements JsonSerializable
+    protected static enum SerializableEnum implements JsonSerializable
     {
         A, B, C;
 
@@ -87,7 +80,7 @@ public class TestEnumSerialization
         }
     }
 
-    protected enum LowerCaseEnum {
+    protected static enum LowerCaseEnum {
         A, B, C;
         private LowerCaseEnum() { }
         @Override
@@ -102,7 +95,6 @@ public class TestEnumSerialization
         }
     }
 
-    // [JACKSON-757]
     static enum NOT_OK {
         V1("v1"); 
         protected String key;
@@ -115,34 +107,8 @@ public class TestEnumSerialization
         protected String key;
         OK(String key) { this.key = key; }
     }
-    
-    // Types for [https://github.com/FasterXML/jackson-databind/issues/24]
-    // (Enums as JSON Objects)
 
-    @JsonFormat(shape=JsonFormat.Shape.OBJECT)
-    static enum PoNUM {
-        A("a1"), B("b2");
-
-        @JsonProperty
-        protected final String value;
-        
-        private PoNUM(String v) { value = v; }
-
-        public String getValue() { return value; }
-    }
-
-    static class PoNUMContainer {
-        @JsonFormat(shape=Shape.NUMBER)
-        public OK text = OK.V1;
-    }
-    
-    @JsonFormat(shape=JsonFormat.Shape.ARRAY) // not supported as of now
-    static enum BrokenPoNum
-    {
-        A, B;
-    }
-
-    @SuppressWarnings("rawtypes")
+    @SuppressWarnings({ "rawtypes", "serial" })
     static class LowerCasingEnumSerializer extends StdSerializer<Enum>
     {
         public LowerCasingEnumSerializer() { super(Enum.class); }
@@ -152,26 +118,37 @@ public class TestEnumSerialization
             jgen.writeString(value.name().toLowerCase());
         }
     }
-    
+
+    protected static enum LC749Enum {
+        A, B, C;
+        private LC749Enum() { }
+        @Override
+        public String toString() { return name().toLowerCase(); }
+    }
+
+    // for [databind#1322]
+    protected enum EnumWithJsonProperty {
+        @JsonProperty("aleph")
+        A;
+    }
+
     /*
-    /**********************************************************
-    /* Tests
-    /**********************************************************
+    /**********************************************************************
+    /* Test methods
+    /**********************************************************************
      */
 
-    private final ObjectMapper mapper = new ObjectMapper();
+    private final ObjectMapper MAPPER = newJsonMapper();
     
     public void testSimple() throws Exception
     {
-        assertEquals("\"B\"", mapper.writeValueAsString(TestEnum.B));
+        assertEquals("\"B\"", MAPPER.writeValueAsString(TestEnum.B));
     }
 
     public void testEnumSet() throws Exception
     {
-        StringWriter sw = new StringWriter();
-        EnumSet<TestEnum> value = EnumSet.of(TestEnum.B);
-        mapper.writeValue(sw, value);
-        assertEquals("[\"B\"]", sw.toString());
+        final EnumSet<TestEnum> value = EnumSet.of(TestEnum.B);
+        assertEquals("[\"B\"]", MAPPER.writeValueAsString(value));
     }
 
     /**
@@ -181,69 +158,65 @@ public class TestEnumSerialization
      */
     public void testEnumUsingToString() throws Exception
     {
-        StringWriter sw = new StringWriter();
-        mapper.writeValue(sw, AnnotatedTestEnum.C2);
-        assertEquals("\"c2\"", sw.toString());
+        assertEquals("\"c2\"", MAPPER.writeValueAsString(AnnotatedTestEnum.C2));
     }
 
-    // Test [JACKSON-214]
     public void testSubclassedEnums() throws Exception
     {
-        assertEquals("\"B\"", mapper.writeValueAsString(EnumWithSubClass.B));
+        assertEquals("\"B\"", MAPPER.writeValueAsString(EnumWithSubClass.B));
     }
 
-    // [JACKSON-193]
-    public void testEnumsWithJsonValue() throws Exception
-    {
-        assertEquals("\"bar\"", mapper.writeValueAsString(EnumWithJsonValue.B));
+    public void testEnumsWithJsonValue() throws Exception {
+        assertEquals("\"value:bar\"", MAPPER.writeValueAsString(EnumWithJsonValue.B));
     }
 
-    // also, for [JACKSON-193], needs to work via mix-ins
     public void testEnumsWithJsonValueUsingMixin() throws Exception
     {
         // can't share, as new mix-ins are added
         ObjectMapper m = new ObjectMapper();
-        m.addMixInAnnotations(TestEnum.class, ToStringMixin.class);
+        m.addMixIn(TestEnum.class, ToStringMixin.class);
         assertEquals("\"b\"", m.writeValueAsString(TestEnum.B));
     }
 
+    // [databind#601]
+    public void testEnumsWithJsonValueInMap() throws Exception
+    {
+        EnumMap<EnumWithJsonValue,String> input = new EnumMap<EnumWithJsonValue,String>(EnumWithJsonValue.class);
+        input.put(EnumWithJsonValue.B, "x");
+        // 24-Sep-2015, tatu: SHOULD actually use annotated method, as per:
+        assertEquals("{\"value:bar\":\"x\"}", MAPPER.writeValueAsString(input));
+    }
+
     /**
      * Test for ensuring that @JsonSerializable is used with Enum types as well
      * as with any other types.
      */
     public void testSerializableEnum() throws Exception
     {
-        assertEquals("\"foo\"", mapper.writeValueAsString(SerializableEnum.A));
+        assertEquals("\"foo\"", MAPPER.writeValueAsString(SerializableEnum.A));
     }
 
-    // [JACKSON-212]
     public void testToStringEnum() throws Exception
     {
         ObjectMapper m = new ObjectMapper();
         m.configure(SerializationFeature.WRITE_ENUMS_USING_TO_STRING, true);
         assertEquals("\"b\"", m.writeValueAsString(LowerCaseEnum.B));
+
+        // [databind#749] but should also be able to dynamically disable
+        assertEquals("\"B\"",
+                m.writer().without(SerializationFeature.WRITE_ENUMS_USING_TO_STRING)
+                    .writeValueAsString(LowerCaseEnum.B));
     }
 
-    // [JACKSON-212]
     public void testToStringEnumWithEnumMap() throws Exception
     {
         ObjectMapper m = new ObjectMapper();
-        m.configure(SerializationFeature.WRITE_ENUMS_USING_TO_STRING, true);
+        m.enable(SerializationFeature.WRITE_ENUMS_USING_TO_STRING);
         EnumMap<LowerCaseEnum,String> enums = new EnumMap<LowerCaseEnum,String>(LowerCaseEnum.class);
         enums.put(LowerCaseEnum.C, "value");
         assertEquals("{\"c\":\"value\"}", m.writeValueAsString(enums));
     }
 
-    // [JACKSON-576]
-    public void testMapWithEnumKeys() throws Exception
-    {
-        MapBean bean = new MapBean();
-        bean.add(TestEnum.B, 3);
-        String json = mapper.writeValueAsString(bean);
-        assertEquals("{\"map\":{\"b\":3}}", json);
-    }
-
-    // [JACKSON-684]
     public void testAsIndex() throws Exception
     {
         // By default, serialize using name
@@ -256,35 +229,14 @@ public class TestEnumSerialization
         assertEquals("1", m.writeValueAsString(TestEnum.B));
     }
 
-    // [JACKSON-757]
     public void testAnnotationsOnEnumCtor() throws Exception
     {
-        assertEquals(quote("V1"), mapper.writeValueAsString(OK.V1));
-        assertEquals(quote("V1"), mapper.writeValueAsString(NOT_OK.V1));
-        assertEquals(quote("V2"), mapper.writeValueAsString(NOT_OK2.V2));
+        assertEquals(quote("V1"), MAPPER.writeValueAsString(OK.V1));
+        assertEquals(quote("V1"), MAPPER.writeValueAsString(NOT_OK.V1));
+        assertEquals(quote("V2"), MAPPER.writeValueAsString(NOT_OK2.V2));
     }
 
-    // Tests for [issue#24]
-
-    public void testEnumAsObjectValid() throws Exception {
-        assertEquals("{\"value\":\"a1\"}", mapper.writeValueAsString(PoNUM.A));
-    }
-
-    public void testEnumAsIndexViaAnnotations() throws Exception {
-        assertEquals("{\"text\":0}", mapper.writeValueAsString(new PoNUMContainer()));
-    }
-    
-    public void testEnumAsObjectBroken() throws Exception
-    {
-        try {
-            String json = mapper.writeValueAsString(BrokenPoNum.A);
-            fail("Should not have succeeded, produced: "+json);
-        } catch (JsonMappingException e) {
-            verifyException(e, "Unsupported serialization shape");
-        }
-    }
-    
-    // [Issue#227]
+    // [databind#227]
     public void testGenericEnumSerializer() throws Exception
     {
         // By default, serialize using name
@@ -294,6 +246,58 @@ public class TestEnumSerialization
         m.registerModule(module);
         assertEquals(quote("b"), m.writeValueAsString(TestEnum.B));
     }
+
+    // [databind#749]
+
+    public void testEnumMapSerDefault() throws Exception {
+        final ObjectMapper mapper = newJsonMapper();
+        EnumMap<LC749Enum, String> m = new EnumMap<LC749Enum, String>(LC749Enum.class);
+        m.put(LC749Enum.A, "value");
+        assertEquals("{\"A\":\"value\"}", mapper.writeValueAsString(m));
+    }
+    
+    public void testEnumMapSerDisableToString() throws Exception {
+        final ObjectMapper mapper = new ObjectMapper();
+        ObjectWriter w = mapper.writer().without(SerializationFeature.WRITE_ENUMS_USING_TO_STRING);
+        EnumMap<LC749Enum, String> m = new EnumMap<LC749Enum, String>(LC749Enum.class);
+        m.put(LC749Enum.A, "value");
+        assertEquals("{\"A\":\"value\"}", w.writeValueAsString(m));
+    }
+
+    public void testEnumMapSerEnableToString() throws Exception {
+        final ObjectMapper mapper = new ObjectMapper();
+        ObjectWriter w = mapper.writer().with(SerializationFeature.WRITE_ENUMS_USING_TO_STRING);
+        EnumMap<LC749Enum, String> m = new EnumMap<LC749Enum, String>(LC749Enum.class);
+        m.put(LC749Enum.A, "value");
+        assertEquals("{\"a\":\"value\"}", w.writeValueAsString(m));
+    }
+
+    // [databind#1322]
+    public void testEnumsWithJsonProperty() throws Exception {
+        assertEquals(quote("aleph"), MAPPER.writeValueAsString(EnumWithJsonProperty.A));
+    }
+
+    // [databind#1535]
+    public void testEnumKeysWithJsonProperty() throws Exception {
+        Map<EnumWithJsonProperty,Integer> input = new HashMap<EnumWithJsonProperty,Integer>();
+        input.put(EnumWithJsonProperty.A, 13);
+        assertEquals(aposToQuotes("{'aleph':13}"), MAPPER.writeValueAsString(input));
+    }
+
+    // [databind#1322]
+    public void testEnumsWithJsonPropertyInSet() throws Exception
+    {
+        assertEquals("[\"aleph\"]",
+                MAPPER.writeValueAsString(EnumSet.of(EnumWithJsonProperty.A)));
+    }
+
+    // [databind#1322]
+    public void testEnumsWithJsonPropertyAsKey() throws Exception
+    {
+        EnumMap<EnumWithJsonProperty,String> input = new EnumMap<EnumWithJsonProperty,String>(EnumWithJsonProperty.class);
+        input.put(EnumWithJsonProperty.A, "b");
+        assertEquals("{\"aleph\":\"b\"}", MAPPER.writeValueAsString(input));
+    }
 }
 
 // [JACKSON-757], non-inner enum
