@@ -102,8 +102,8 @@ public class TestJsonSerialize
     }
 
     static class Bar294{
-        @JsonProperty private String id;
-        @JsonProperty private String name;
+        @JsonProperty protected String id;
+        @JsonProperty protected String name;
 
         public Bar294() { }
         public Bar294(String id) {
@@ -148,8 +148,9 @@ public class TestJsonSerialize
     {
         try {
             serializeAsString(MAPPER, new BrokenClass());
+            fail("Should not succeed");
         } catch (Exception e) {
-            verifyException(e, "not a super-type of");
+            verifyException(e, "types not related");
         }
     }
 
@@ -187,8 +188,9 @@ public class TestJsonSerialize
 
     public void testStaticTypingWithMap() throws Exception
     {
-        ObjectMapper m = new ObjectMapper();
-        m.configure(MapperFeature.USE_STATIC_TYPING, true);
+        ObjectMapper m = jsonMapperBuilder()
+                .configure(MapperFeature.USE_STATIC_TYPING, true)
+                .build();
         ValueMap map = new ValueMap();
         map.put("a", new ValueClass());
         assertEquals("{\"a\":{\"x\":3}}", serializeAsString(m, map));
@@ -196,8 +198,9 @@ public class TestJsonSerialize
 
     public void testStaticTypingWithArrayList() throws Exception
     {
-        ObjectMapper m = new ObjectMapper();
-        m.configure(MapperFeature.USE_STATIC_TYPING, true);
+        ObjectMapper m = jsonMapperBuilder()
+                .configure(MapperFeature.USE_STATIC_TYPING, true)
+                .build();
         ValueList list = new ValueList();
         list.add(new ValueClass());
         assertEquals("[{\"x\":3}]", m.writeValueAsString(list));
@@ -205,8 +208,9 @@ public class TestJsonSerialize
 
     public void testStaticTypingWithLinkedList() throws Exception
     {
-        ObjectMapper m = new ObjectMapper();
-        m.configure(MapperFeature.USE_STATIC_TYPING, true);
+        ObjectMapper m = jsonMapperBuilder()
+                .configure(MapperFeature.USE_STATIC_TYPING, true)
+                .build();
         ValueLinkedList list = new ValueLinkedList();
         list.add(new ValueClass());
         assertEquals("[{\"x\":3}]", serializeAsString(m, list));
@@ -214,8 +218,9 @@ public class TestJsonSerialize
     
     public void testStaticTypingWithArray() throws Exception
     {
-        ObjectMapper m = new ObjectMapper();
-        m.configure(MapperFeature.USE_STATIC_TYPING, true);
+        ObjectMapper m = jsonMapperBuilder()
+                .configure(MapperFeature.USE_STATIC_TYPING, true)
+                .build();
         ValueInterface[] array = new ValueInterface[] { new ValueClass() };
         assertEquals("[{\"x\":3}]", serializeAsString(m, array));
     }
