@@ -56,7 +56,7 @@ public class TestRootType
         public int a = 3;
     }
 
-    // [Issue#412]
+    // [databind#412]
     static class TestCommandParent {
         public String uuid;
         public int type;
@@ -90,7 +90,7 @@ public class TestRootType
         assertEquals(Boolean.TRUE, result.get("b2"));
 
         // and then using specified typed writer
-        ObjectWriter w = mapper.writerWithType(BaseType.class);
+        ObjectWriter w = mapper.writerFor(BaseType.class);
         String json = w.writeValueAsString(bean);
         result = (Map<String,Object>)mapper.readValue(json, Map.class);
         assertEquals(2, result.size());
@@ -104,7 +104,7 @@ public class TestRootType
         SubType bean = new SubType();
 
         // let's constrain by interface:
-        ObjectWriter w = mapper.writerWithType(BaseInterface.class);
+        ObjectWriter w = mapper.writerFor(BaseInterface.class);
         String json = w.writeValueAsString(bean);
         @SuppressWarnings("unchecked")
         Map<String,Object> result = mapper.readValue(json, Map.class);
@@ -114,11 +114,12 @@ public class TestRootType
 
     public void testInArray() throws Exception
     {
-        ObjectMapper mapper = new ObjectMapper();
+        ObjectMapper mapper = jsonMapperBuilder()
         // must force static typing, otherwise won't matter a lot
-        mapper.configure(MapperFeature.USE_STATIC_TYPING, true);
+                .configure(MapperFeature.USE_STATIC_TYPING, true)
+                .build();
         SubType[] ob = new SubType[] { new SubType() };
-        String json = mapper.writerWithType(BaseInterface[].class).writeValueAsString(ob);
+        String json = mapper.writerFor(BaseInterface[].class).writeValueAsString(ob);
         // should propagate interface type through due to root declaration; static typing
         assertEquals("[{\"b\":3}]", json);
     }
@@ -133,16 +134,23 @@ public class TestRootType
         SubType bean = new SubType();
 
         // and then let's try using incompatible type
-        ObjectWriter w = mapper.writerWithType(HashMap.class);
+        ObjectWriter w = mapper.writerFor(HashMap.class);
         try {
             w.writeValueAsString(bean);
             fail("Should have failed due to incompatible type");
         } catch (JsonProcessingException e) {
             verifyException(e, "Incompatible types");
         }
+
+        // and also with alternate output method
+        try {
+            w.writeValueAsBytes(bean);
+            fail("Should have failed due to incompatible type");
+        } catch (JsonProcessingException e) {
+            verifyException(e, "Incompatible types");
+        }
     }
-    
-    // [JACKSON-398]
+
     public void testJackson398() throws Exception
     {
         ObjectMapper mapper = objectMapper();
@@ -153,12 +161,12 @@ public class TestRootType
         final String EXP = "[{\"beanClass\":\"TestRootType$TestClass398\",\"property\":\"aa\"}]";
         
         // First simplest way:
-        String json = mapper.writerWithType(collectionType).writeValueAsString(typedList);
+        String json = mapper.writerFor(collectionType).writeValueAsString(typedList);
         assertEquals(EXP, json);
 
         StringWriter out = new StringWriter();
         JsonFactory f = new JsonFactory();
-        mapper.writerWithType(collectionType).writeValue(f.createGenerator(out), typedList);
+        mapper.writerFor(collectionType).writeValue(f.createGenerator(out), typedList);
 
         assertEquals(EXP, out.toString());
     }
@@ -179,27 +187,26 @@ public class TestRootType
     public void testIssue456WrapperPart() throws Exception
     {
         ObjectMapper mapper = objectMapper();
-        assertEquals("123", mapper.writerWithType(Integer.TYPE).writeValueAsString(Integer.valueOf(123)));
-        assertEquals("456", mapper.writerWithType(Long.TYPE).writeValueAsString(Long.valueOf(456L)));
+        assertEquals("123", mapper.writerFor(Integer.TYPE).writeValueAsString(Integer.valueOf(123)));
+        assertEquals("456", mapper.writerFor(Long.TYPE).writeValueAsString(Long.valueOf(456L)));
     }
 
-    // [JACKSON-630] also, allow annotation to define root name
     public void testRootNameAnnotation() throws Exception
     {
         String json = WRAP_ROOT_MAPPER.writeValueAsString(new WithRootName());
         assertEquals("{\"root\":{\"a\":3}}", json);
     }
 
-    // [Issue#412]
+    // [databind#412]
     public void testRootNameWithExplicitType() throws Exception
     {
         TestCommandChild cmd = new TestCommandChild();
         cmd.uuid = "1234";
         cmd.type = 1;
 
-        ObjectWriter writer = WRAP_ROOT_MAPPER.writerWithType(TestCommandParent.class);
+        ObjectWriter writer = WRAP_ROOT_MAPPER.writerFor(TestCommandParent.class);
         String json =  writer.writeValueAsString(cmd);
 
-        assertEquals(json, "{\"TestCommandParent\":{\"uuid\":\"1234\",\"type\":1}}");
+        assertEquals("{\"TestCommandParent\":{\"uuid\":\"1234\",\"type\":1}}", json);
     }
 }
