@@ -3,6 +3,7 @@ package com.fasterxml.jackson.databind.type;
 import java.util.*;
 
 import com.fasterxml.jackson.annotation.JsonTypeInfo;
+import com.fasterxml.jackson.core.type.TypeReference;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.jsontype.TypeResolverBuilder;
 import com.fasterxml.jackson.databind.jsontype.impl.StdTypeResolverBuilder;
@@ -41,5 +42,13 @@ public class RecursiveType1658Test extends BaseMapTest
         Tree<?> tRead = mapper.readValue(res, Tree.class);
 
         assertNotNull(tRead);
+
+        // 30-Oct-2019, tatu: Let's actually verify that description will be safe to use, too
+        JavaType resolved = mapper.getTypeFactory()
+                .constructType(new TypeReference<Tree<String>> () { });
+        final String namePath = Tree.class.getName().replace('.', '/');
+        assertEquals("L"+namePath+";", resolved.getErasedSignature());
+        assertEquals("L"+namePath+"<Ljava/lang/String;L"+namePath+";>;",
+                resolved.getGenericSignature());
     }
 }
