@@ -232,8 +232,14 @@ public class TestTypeFactory
                 can);
         assertEquals(t, tf.constructFromCanonical(can));
 
-        // [databind#1941]: allow "raw" types too
+        // [databind#2109]: also ReferenceTypes
+        t = tf.constructType(new TypeReference<AtomicReference<Long>>() { });
+        can = t.toCanonical();
+        assertEquals("java.util.concurrent.atomic.AtomicReference<java.lang.Long>",
+                can);
+        assertEquals(t, tf.constructFromCanonical(can));
 
+        // [databind#1941]: allow "raw" types too
         t = tf.constructFromCanonical("java.util.List");
         assertEquals(List.class, t.getRawClass());
         assertEquals(CollectionType.class, t.getClass());
@@ -276,6 +282,7 @@ public class TestTypeFactory
         JavaType t = tf.constructType(ArrayList.class);
         assertEquals(CollectionType.class, t.getClass());
         assertSame(ArrayList.class, t.getRawClass());
+        assertSame(Object.class, ((CollectionType) t).getContentType().getRawClass());
 
         // And then the proper way
         t = tf.constructType(new TypeReference<ArrayList<String>>() { });
