@@ -9,19 +9,22 @@ import com.fasterxml.jackson.databind.*;
  */
 public class TestUnknownProperty426 extends BaseMapTest
 {
-    // For [Issue#426]
-    @JsonIgnoreProperties({ "userId"})
-    public class User {
+    // For [databind#426]
+    @JsonIgnoreProperties({ "userId" })
+    static class User {
+        public String firstName;
         Integer userId; 
 
-        void setUserId(String id) {
-            setUserId(new Integer(id));
+        public void setUserId(CharSequence id) {
+            // 21-Dec-2015, tatu: With a fix in 2.7, use of String would not
+            //   trigger the problem, so use CharSequence...
+            setUserId(Integer.valueOf(id.toString()));
         }
-    
+
         public Integer getUserId() {
             return userId;
         }
-    
+
         public void setUserId(Integer v) {
             this.userId = v;
         }
@@ -35,11 +38,13 @@ public class TestUnknownProperty426 extends BaseMapTest
 
     private final ObjectMapper MAPPER = new ObjectMapper();
 
+    // Aside from [databind#426], also relevant: [databind#1044]?
     public void testIssue426() throws Exception
     {
-        String jsonString = "{id: 9, firstName: \"Mike\" }";
-        User result = MAPPER.reader( User.class ).readValue(jsonString);
+        final String JSON = aposToQuotes("{'userId': 9, 'firstName': 'Mike' }");
+        User result = MAPPER.readerFor(User.class).readValue(JSON);
         assertNotNull(result);
+        assertEquals("Mike", result.firstName);
     }
 }
 
